"""战斗状态处理器 — 各状态节点的决策逻辑。


每个 ``_handle_*`` 方法对应一个 :class:`~autowsgr.combat.state.CombatPhase`，
执行该阶段所需的决策和操作，并返回 :class:`~autowsgr.types.ConditionFlag`
指示引擎是否继续循环。
"""

from __future__ import annotations

import time
from typing import TYPE_CHECKING

from autowsgr.combat.actions import (
    check_blood,
    click_enter_fight,
    click_fight_condition,
    click_formation,
    click_image,
    click_night_battle,
    click_proceed,
    click_result,
    click_retreat,
    click_skip_missile_animation,
    detect_result_grade,
    detect_ship_stats,
    get_enemy_formation,
    get_enemy_info,
    get_ship_drop,
    image_exist,
)
from autowsgr.combat.recognition import detect_mvp
from autowsgr.image_resources import TemplateKey
from autowsgr.infra.logger import get_logger
from autowsgr.types import ConditionFlag, Formation, ShipDamageState
from autowsgr.ui.utils import wait_leave_page
from autowsgr.vision import ImageChecker

from .history import CombatEvent, CombatHistory, EventType, FightResult
from .plan import CombatMode, CombatPlan, NodeDecision
from .rules import RuleResult
from .state import CombatPhase


if TYPE_CHECKING:
    from autowsgr.emulator import AndroidController
    from autowsgr.vision import OCREngine


_log = get_logger('combat')


# ── 状态 → 处理器方法名映射  ────────────────────────────────────────────────

_PHASE_HANDLERS: dict[CombatPhase, str] = {
    CombatPhase.FIGHT_CONDITION: '_handle_fight_condition',
    CombatPhase.SPOT_ENEMY_SUCCESS: '_handle_spot_enemy',
    CombatPhase.FORMATION: '_handle_formation',
    CombatPhase.MISSILE_ANIMATION: '_handle_missile_animation',
    CombatPhase.FIGHT_PERIOD: '_handle_fight_period',
    CombatPhase.NIGHT_PROMPT: '_handle_night_prompt',
    CombatPhase.RESULT: '_handle_result',
    CombatPhase.GET_SHIP: '_handle_get_ship',
    CombatPhase.PROCEED: '_handle_proceed',
    CombatPhase.FLAGSHIP_SEVERE_DAMAGE: '_handle_flagship_severe_damage',
    CombatPhase.DOCK_FULL: '_handle_dock_full',
}


class PhaseHandlersMixin:
    """战斗状态处理器 Mixin。

    为 ``CombatEngine`` 提供 ``_make_decision`` 及所有 ``_handle_*`` 方法。

    约定: 本 Mixin 假设宿主类具有以下属性::

        _device: AndroidController
        _plan: CombatPlan
        _ocr: OCREngine | None
        _node: str
        _last_action: str
        _ship_stats: list[ShipDamageState]
        _history: CombatHistory
        _node_count: int
        _formation_by_rule: Formation | None
    """

    # 类型提示 (供 IDE/mypy 在 Mixin 上下文使用)
    _device: AndroidController
    _plan: CombatPlan
    _ocr: OCREngine | None
    _node: str
    _last_action: str
    _ship_stats: list[ShipDamageState]
    _history: CombatHistory
    _node_count: int
    _formation_by_rule: Formation | None

    def _make_decision(self, phase: CombatPhase) -> ConditionFlag:
        """根据当前状态做出决策并执行操作。

        每个状态通过 ``_PHASE_HANDLERS`` 映射到对应的处理器方法；
        处理器执行完毕后，若当前状态是终止阶段则返回 ``FIGHT_END``。

        Parameters
        ----------
        phase:
            当前识别到的战斗状态。

        Returns
        -------
        ConditionFlag
        """
        # ── 派发处理器 ──
        handler_name = _PHASE_HANDLERS.get(phase)
        if handler_name is not None:
            result = getattr(self, handler_name)()
        else:
            result = ConditionFlag.FIGHT_CONTINUE

        # ── 终止态检查 ──
        if phase == self._plan.end_phase:
            self._history.add(
                CombatEvent(
                    event_type=EventType.AUTO_RETURN,
                    node=self._node,
                    action='正常',
                )
            )
            return ConditionFlag.FIGHT_END

        return result

    # ── 各状态处理器 ─────────────────────────────────────────────────────────

    def _handle_fight_condition(self) -> ConditionFlag:
        """处理战况选择。
        TODO: 需测试
        """
        condition = self._plan.fight_condition
        click_fight_condition(self._device, condition)
        self._last_action = str(condition.value)

        self._history.add(
            CombatEvent(
                event_type=EventType.FIGHT_CONDITION,
                node=self._node,
                action=str(condition.value),
            )
        )
        return ConditionFlag.FIGHT_CONTINUE

    def _handle_spot_enemy(self) -> ConditionFlag:
        """处理索敌成功 — 核心决策节点。

        决策顺序:
        1. 采集敌方编成和阵型
        2. 检查节点是否在白名单中
        3. 检查阵型规则 (formation_rules)
        4. 检查敌舰规则 (enemy_rules)
        5. 根据结果执行: 撤退 / 迂回 / 设置阵型 / 进入战斗
        """
        # ── 信息采集 ──
        mode = 'exercise' if self._plan.mode == CombatMode.EXERCISE else 'fight'
        enemies = get_enemy_info(self._device, mode=mode)
        enemy_formation = get_enemy_formation(self._device, self._ocr)
        _log.info('[Combat] 敌方编成: {} 阵型: {}', enemies, enemy_formation)

        decision = self._get_current_decision()

        # 白名单检查
        if not self._plan.is_selected_node(self._node):
            click_retreat(self._device)
            self._last_action = 'retreat'
            self._history.add(
                CombatEvent(
                    event_type=EventType.SPOT_ENEMY,
                    node=self._node,
                    action='撤退',
                    extra={'reason': '不在预设点'},
                )
            )
            return ConditionFlag.FIGHT_END

        # 检查迂回按钮是否可用
        can_detour = image_exist(self._device, TemplateKey.BYPASS, 0.8)
        want_detour = can_detour and decision.detour

        # 阵型规则优先
        rule_action = None
        if decision.formation_rules and enemy_formation:
            rule_action = decision.formation_rules.evaluate_formation(enemy_formation)

        # 敌舰规则
        if (
            rule_action is None or rule_action.result == RuleResult.NO_ACTION
        ) and decision.enemy_rules:
            rule_action = decision.enemy_rules.evaluate(enemies)

        # 应用规则结果
        if rule_action is not None:
            if rule_action.result == RuleResult.RETREAT:
                click_retreat(self._device)
                self._last_action = 'retreat'
                self._history.add(
                    CombatEvent(
                        event_type=EventType.SPOT_ENEMY,
                        node=self._node,
                        action='撤退',
                        enemies=enemies.copy(),
                    )
                )
                return ConditionFlag.FIGHT_END

            if rule_action.result == RuleResult.DETOUR:
                if not can_detour:
                    _log.error('[Combat] 规则指定迂回, 但该点无法迂回')
                    raise ValueError('该点无法迂回, 但在规则中指定了迂回')
                want_detour = True

            if rule_action.result == RuleResult.FORMATION and rule_action.formation:
                self._formation_by_rule = rule_action.formation

        # 执行迂回
        if want_detour:
            clicked = click_image(self._device, TemplateKey.BYPASS, 2.5)
            if clicked:
                _log.info('[Combat] 执行迂回')
                spot_templates = TemplateKey.SPOT_ENEMY.templates
                wait_leave_page(
                    self._device,
                    checker=lambda screen: (
                        ImageChecker.find_any(screen, spot_templates, confidence=0.8) is not None
                    ),
                    timeout=10.0,
                    source='spot_enemy_success',
                    target='map_routing',
                )
            else:
                _log.warning('[Combat] 未找到迂回按钮')
            self._last_action = 'detour'
            self._history.add(
                CombatEvent(
                    event_type=EventType.SPOT_ENEMY,
                    node=self._node,
                    action='迂回',
                    enemies=enemies.copy(),
                )
            )
            return ConditionFlag.FIGHT_CONTINUE

        # 远程导弹支援
        if decision.long_missile_support:
            clicked = click_image(self._device, TemplateKey.MISSILE_SUPPORT, 2.5)
            if clicked:
                _log.info('[Combat] 开启远程导弹支援')
            else:
                _log.warning('[Combat] 未找到远程支援按钮')

        # 进入战斗
        click_enter_fight(self._device)
        self._last_action = 'fight'
        self._history.add(
            CombatEvent(
                event_type=EventType.SPOT_ENEMY,
                node=self._node,
                action='战斗',
                enemies=enemies.copy(),
            )
        )
        return ConditionFlag.FIGHT_CONTINUE

    def _handle_formation(self) -> ConditionFlag:
        """处理阵型选择。"""
        decision = self._get_current_decision()
        is_from_spot_enemy = self._last_action in ('fight', 'detour')

        # 白名单检查
        if not self._plan.is_selected_node(self._node):
            self._history.add(
                CombatEvent(
                    event_type=EventType.FORMATION,
                    node=self._node,
                    action='SL',
                    extra={'reason': '不在预设点'},
                )
            )
            return ConditionFlag.SL

        # 迂回失败 SL
        if is_from_spot_enemy and self._last_action == 'detour' and decision.SL_when_detour_fails:
            self._history.add(
                CombatEvent(
                    event_type=EventType.DETOUR,
                    node=self._node,
                    result='失败',
                )
            )
            self._history.add(
                CombatEvent(
                    event_type=EventType.FORMATION,
                    node=self._node,
                    action='SL',
                )
            )
            return ConditionFlag.SL

        # 确定阵型
        formation = decision.formation

        if is_from_spot_enemy and self._formation_by_rule is not None:
            formation = self._formation_by_rule
            self._formation_by_rule = None
            _log.debug('[Combat] 使用规则阵型: {}', formation.name)
        elif not is_from_spot_enemy:
            # 索敌失败
            if decision.SL_when_spot_enemy_fails:
                self._history.add(
                    CombatEvent(
                        event_type=EventType.FORMATION,
                        node=self._node,
                        action='SL',
                        extra={'reason': '索敌失败'},
                    )
                )
                return ConditionFlag.SL
            if decision.formation_when_spot_enemy_fails is not None:
                formation = decision.formation_when_spot_enemy_fails

        # 选择阵型
        _log.info('[Combat] 阵型选择: {}', formation.name)
        click_formation(self._device, formation)

        self._last_action = str(formation.value)
        self._history.add(
            CombatEvent(
                event_type=EventType.FORMATION,
                node=self._node,
                action=f'阵型{formation.value} ({formation.name})',
            )
        )
        return ConditionFlag.FIGHT_CONTINUE

    def _handle_missile_animation(self) -> ConditionFlag:
        """跳过导弹支援动画。"""
        _log.info('[Combat] 跳过导弹支援动画')
        click_skip_missile_animation(self._device)
        self._last_action = 'skip_animation'
        return ConditionFlag.FIGHT_CONTINUE

    def _handle_fight_period(self) -> ConditionFlag:
        """处理战斗进行中。"""
        decision = self._get_current_decision()
        if decision.SL_when_enter_fight:
            self._history.add(
                CombatEvent(
                    event_type=EventType.ENTER_FIGHT,
                    node=self._node,
                    action='SL',
                )
            )
            return ConditionFlag.SL
        return ConditionFlag.FIGHT_CONTINUE

    def _handle_night_prompt(self) -> ConditionFlag:
        """处理夜战选择。"""
        decision = self._get_current_decision()
        pursue = decision.night

        _log.info('[Combat] 夜战选择: {}', '追击' if pursue else '撤退')
        click_night_battle(self._device, pursue=pursue)
        self._last_action = 'yes' if pursue else 'no'

        self._history.add(
            CombatEvent(
                event_type=EventType.NIGHT_BATTLE,
                node=self._node,
                action='追击' if pursue else '撤退',
            )
        )
        return ConditionFlag.FIGHT_CONTINUE

    def _handle_result(self) -> ConditionFlag:
        """处理战果结算 -- 识别评级、更新血量、MVP、关闭界面。"""
        # ── 信息采集 ──
        grade = detect_result_grade(self._device)
        self._ship_stats = detect_ship_stats(self._device, self._ship_stats)

        # MVP 识别 (在关闭结算界面之前)
        screen = self._device.screenshot()
        mvp = detect_mvp(screen)

        fight_result = FightResult(
            node=self._node,
            mvp=mvp,
            grade=grade,
            ship_stats=self._ship_stats[:],
        )
        self._history.add(
            CombatEvent(
                event_type=EventType.RESULT,
                node=self._node,
                result=grade,
                ship_stats=self._ship_stats[:],
                extra={'mvp': mvp},
            )
        )
        _log.info('[Combat] 战果: {} 节点: {}', fight_result, self._node)

        # ── 关闭结算界面 ──
        time.sleep(1)
        click_result(self._device)
        time.sleep(0.25)
        click_result(self._device)
        return ConditionFlag.FIGHT_CONTINUE

    def _handle_get_ship(self) -> ConditionFlag:
        """处理获取舰船。"""
        ship_name = get_ship_drop(self._device)
        if ship_name:
            _log.info('[Combat] 获得舰船: {}', ship_name)

        self._history.add(
            CombatEvent(
                event_type=EventType.GET_SHIP,
                node=self._node,
                result=ship_name or '',
            )
        )
        click_result(self._device)
        return ConditionFlag.FIGHT_CONTINUE

    def _handle_proceed(self) -> ConditionFlag:
        """处理继续前进 / 回港决策。

        决策依据:
        1. 当前节点的 ``proceed`` 配置
        2. 血量是否满足 ``proceed_stop`` 条件
        """
        self._node_count += 1
        decision = self._get_current_decision()

        should_proceed = decision.proceed and check_blood(self._ship_stats, decision.proceed_stop)

        _log.info('[Combat] 继续前进决策: {}', '前进' if should_proceed else '回港')
        click_proceed(self._device, go_forward=should_proceed)
        self._last_action = 'yes' if should_proceed else 'no'

        self._history.add(
            CombatEvent(
                event_type=EventType.PROCEED,
                node=self._node,
                action='前进' if should_proceed else '回港',
                ship_stats=self._ship_stats[:],
            )
        )

        if should_proceed:
            return ConditionFlag.FIGHT_CONTINUE
        return ConditionFlag.FIGHT_END

    def _handle_flagship_severe_damage(self) -> ConditionFlag:
        """处理旗舰大破。"""
        _log.info('[Combat] 旗舰大破, 强制回港')
        click_image(self._device, TemplateKey.FLAGSHIP_DAMAGE, 2.0)
        time.sleep(0.25)

        self._history.add(
            CombatEvent(
                event_type=EventType.FLAGSHIP_DAMAGE,
                node=self._node,
                action='回港',
            )
        )
        return ConditionFlag.FIGHT_END

    def _handle_dock_full(self) -> ConditionFlag:
        """处理船坞已满弹窗 — 返回 DOCK_FULL 标志交由上层处理。"""
        _log.warning('[Combat] 检测到船坞已满，战斗无法开始')
        self._history.add(
            CombatEvent(
                event_type=EventType.AUTO_RETURN,
                node=self._node,
                action='船坞已满',
            )
        )
        return ConditionFlag.DOCK_FULL

    # ── Mixin 所需的方法签名 (由宿主类提供) ──

    def _get_current_decision(self) -> NodeDecision:
        """获取当前节点的决策 (由 CombatEngine 实现)。"""
        raise NotImplementedError



"""任务执行路由 — /api/task/*"""

from __future__ import annotations

from typing import Annotated, Any

from fastapi import APIRouter, HTTPException
from pydantic import Discriminator

from autowsgr.infra.logger import get_logger
from autowsgr.server.schemas import (
    ApiResponse,
    CampaignRequest,
    DecisiveRequest,
    EventFightRequest,
    ExerciseRequest,
    NormalFightRequest,
)
from autowsgr.server.serializers import build_combat_plan, convert_combat_result
from autowsgr.server.task_manager import task_manager

from ..main import get_context


_log = get_logger('server')

router = APIRouter(prefix='/api/task', tags=['task'])


TaskRequestUnion = Annotated[
    NormalFightRequest | EventFightRequest | CampaignRequest | ExerciseRequest | DecisiveRequest,
    Discriminator('type'),
]


@router.post('/start', response_model=ApiResponse)
async def task_start(request: TaskRequestUnion):  # type: ignore
    """启动任务 (异步执行，立即返回)。"""
    if task_manager.is_running:
        raise HTTPException(status_code=409, detail='已有任务正在运行')

    try:
        ctx = get_context()
    except RuntimeError as e:
        raise HTTPException(status_code=503, detail=str(e))

    ctx.stop_event = task_manager._stop_event

    if isinstance(request, NormalFightRequest):
        return await _start_normal_fight(ctx, request)
    elif isinstance(request, EventFightRequest):
        return await _start_event_fight(ctx, request)
    elif isinstance(request, CampaignRequest):
        return await _start_campaign(ctx, request)
    elif isinstance(request, ExerciseRequest):
        return await _start_exercise(ctx, request)
    elif isinstance(request, DecisiveRequest):
        return await _start_decisive(ctx, request)
    else:
        raise HTTPException(status_code=400, detail='未知的任务类型')


@router.post('/stop', response_model=ApiResponse)
async def task_stop():
    """停止当前任务。"""
    if not task_manager.is_running:
        return ApiResponse(success=True, message='没有正在运行的任务')

    success = task_manager.stop_task()
    if success:
        return ApiResponse(
            success=True,
            data={
                'task_id': task_manager.current_task.task_id,
                'status': 'stopped',
            },
            message='已请求停止任务',
        )
    else:
        return ApiResponse(success=False, error='停止失败')


@router.get('/status', response_model=ApiResponse)
async def task_status():
    """查询当前任务状态。"""
    status = task_manager.get_status()
    return ApiResponse(success=True, data=status)


# ═══════════════════════════════════════════════════════════════════════════════
# 任务启动辅助
# ═══════════════════════════════════════════════════════════════════════════════


async def _start_normal_fight(ctx: Any, request: NormalFightRequest) -> ApiResponse:
    """启动常规战任务。"""
    from autowsgr.combat import CombatPlan
    from autowsgr.ops import run_normal_fight

    def executor(task_info: Any) -> list[dict[str, Any]]:
        results = []

        if request.plan_id:
            plan = CombatPlan.from_yaml(request.plan_id)
        elif request.plan:
            plan = build_combat_plan(request.plan)
        else:
            raise ValueError('必须提供 plan 或 plan_id')

        for i in range(request.times):
            if task_manager.should_stop():
                break

            task_manager.update_progress(current_round=i + 1)
            _log.info('[Task] 常规战第 {}/{} 轮', i + 1, request.times)

            try:
                result = run_normal_fight(ctx, plan, times=1)[0]
                results.append(convert_combat_result(result, i + 1))
                task_manager.add_result(results[-1])
            except Exception as e:
                _log.error('[Task] 第 {} 轮失败: {}', i + 1, e)
                results.append({'round': i + 1, 'success': False, 'error': str(e)})

        return results

    task_id = task_manager.start_task(
        task_type='normal_fight',
        total_rounds=request.times,
        executor=executor,
    )

    return ApiResponse(
        success=True,
        data={'task_id': task_id, 'status': 'running'},
        message='任务已启动',
    )


async def _start_event_fight(ctx: Any, request: EventFightRequest) -> ApiResponse:
    """启动活动战任务。"""
    from autowsgr.combat import CombatPlan
    from autowsgr.ops import run_event_fight

    def executor(task_info: Any) -> list[dict[str, Any]]:
        results = []

        if request.plan_id:
            plan = CombatPlan.from_yaml(request.plan_id)
        elif request.plan:
            plan = build_combat_plan(request.plan)
        else:
            raise ValueError('必须提供 plan 或 plan_id')

        fleet_id = request.fleet_id or plan.fleet_id

        for i in range(request.times):
            if task_manager.should_stop():
                break

            task_manager.update_progress(current_round=i + 1)
            _log.info('[Task] 活动战第 {}/{} 轮', i + 1, request.times)

            try:
                result = run_event_fight(ctx, plan, times=1, fleet_id=fleet_id)[0]
                results.append(convert_combat_result(result, i + 1))
                task_manager.add_result(results[-1])
            except Exception as e:
                _log.error('[Task] 第 {} 轮失败: {}', i + 1, e)
                results.append({'round': i + 1, 'success': False, 'error': str(e)})

        return results

    task_id = task_manager.start_task(
        task_type='event_fight',
        total_rounds=request.times,
        executor=executor,
    )

    return ApiResponse(
        success=True,
        data={'task_id': task_id, 'status': 'running'},
        message='任务已启动',
    )


async def _start_campaign(ctx: Any, request: CampaignRequest) -> ApiResponse:
    """启动战役任务。"""
    from autowsgr.ops import CampaignRunner

    def executor(task_info: Any) -> list[dict[str, Any]]:
        runner = CampaignRunner(
            ctx,
            campaign_name=request.campaign_name,
            times=1,
        )

        results = []
        for i in range(request.times):
            if task_manager.should_stop():
                break

            task_manager.update_progress(current_round=i + 1)
            _log.info('[Task] 战役第 {}/{} 轮', i + 1, request.times)

            try:
                result = runner.run()
                for j, r in enumerate(result):
                    converted = convert_combat_result(r, i * len(result) + j + 1)
                    results.append(converted)
                    task_manager.add_result(converted)
            except Exception as e:
                _log.error('[Task] 第 {} 轮失败: {}', i + 1, e)
                results.append({'round': i + 1, 'success': False, 'error': str(e)})

        return results

    task_id = task_manager.start_task(
        task_type='campaign',
        total_rounds=request.times,
        executor=executor,
    )

    return ApiResponse(
        success=True,
        data={'task_id': task_id, 'status': 'running'},
        message='任务已启动',
    )


async def _start_exercise(ctx: Any, request: ExerciseRequest) -> ApiResponse:
    """启动演习任务。"""
    from autowsgr.ops import ExerciseRunner

    def executor(task_info: Any) -> list[dict[str, Any]]:
        runner = ExerciseRunner(ctx, fleet_id=request.fleet_id)
        task_manager.update_progress(current_round=1, current_node='演习')

        try:
            results = runner.run()
            return [convert_combat_result(r, i + 1) for i, r in enumerate(results)]
        except Exception as e:
            return [{'round': 1, 'success': False, 'error': str(e)}]

    task_id = task_manager.start_task(
        task_type='exercise',
        total_rounds=1,
        executor=executor,
    )

    return ApiResponse(
        success=True,
        data={'task_id': task_id, 'status': 'running'},
        message='任务已启动',
    )


async def _start_decisive(ctx: Any, request: DecisiveRequest) -> ApiResponse:
    """启动决战任务。"""
    from autowsgr.infra import DecisiveConfig
    from autowsgr.ops import DecisiveController

    def executor(task_info: Any) -> list[dict[str, Any]]:
        config = DecisiveConfig(
            chapter=request.chapter,
            level1=request.level1,
            level2=request.level2,
            flagship_priority=request.flagship_priority,
            use_quick_repair=request.use_quick_repair,
        )

        controller = DecisiveController(ctx, config)
        task_manager.update_progress(current_round=1, current_node='决战')

        try:
            result = controller.run()
            return [{'round': 1, 'success': True, 'result': result.value}]
        except Exception as e:
            return [{'round': 1, 'success': False, 'error': str(e)}]

    task_id = task_manager.start_task(
        task_type='decisive',
        total_rounds=1,
        executor=executor,
    )

    return ApiResponse(
        success=True,
        data={'task_id': task_id, 'status': 'running'},
        message='任务已启动',
    )



"""出征准备页面 UI 控制器端到端测试。

运行方式::

    python testing/ui/battle_preparation/e2e.py [serial] [--auto] [--debug]

前置条件：
    游戏位于 **出征准备页面** (地图 → 普通关卡 → 选择关卡后进入)

测试内容：
    1. 验证初始状态（当前舰队、当前面板、自动补给）
    2. 依次切换舰队: 1 → 2 → 3 → 4 → 1
    3. 依次切换底部面板: QUICK_SUPPLY → QUICK_REPAIR → EQUIPMENT → STATS
    4. 拨动「自动补给」两次（开 → 关，或 关 → 开 → 关）
    5. 出征准备页面 → ◁ 地图/上级页面
"""

from __future__ import annotations

import sys
from pathlib import Path


sys.path.insert(0, str(Path(__file__).parent.parent.parent.parent))

from testing.ui._framework import (
    UIControllerTestRunner,
    connect_via_launcher,
    ensure_page,
    info,
    parse_e2e_args,
    reset_to_main_page,
)


def run_test(runner: UIControllerTestRunner) -> None:
    from autowsgr.ui.battle.preparation import BattlePreparationPage, Panel

    bp_page = BattlePreparationPage(runner.ctrl)

    runner.verify_current(
        '初始验证: 出征准备页面', '出征准备页面', BattlePreparationPage.is_current_page
    )
    if runner.aborted:
        return

    runner.read_state(
        '出征准备页面',
        readers={
            '当前舰队': lambda s: BattlePreparationPage.get_selected_fleet(s),
            '当前面板': lambda s: BattlePreparationPage.get_active_panel(s),
            '自动补给': lambda s: BattlePreparationPage.is_auto_supply_enabled(s),
        },
    )

    # 舰队切换
    for fleet_id in [2, 3, 4, 1]:
        runner.execute_step(
            f'切换舰队 → 第{fleet_id}舰队',
            '出征准备页面',
            BattlePreparationPage.is_current_page,
            lambda fid=fleet_id: bp_page.select_fleet(fid),
        )
        if runner.aborted:
            return

    # 面板切换
    for panel in [Panel.QUICK_SUPPLY, Panel.QUICK_REPAIR, Panel.EQUIPMENT, Panel.STATS]:
        runner.execute_step(
            f'切换面板 → {panel.value}',
            '出征准备页面',
            BattlePreparationPage.is_current_page,
            lambda p=panel: bp_page.select_panel(p),
        )
        if runner.aborted:
            return

    # 自动补给拨动两次
    for i in range(1, 3):
        runner.execute_step(
            f'拨动自动补给 (第{i}次)',
            '出征准备页面',
            BattlePreparationPage.is_current_page,
            lambda: bp_page.toggle_auto_supply(),
        )
        if runner.aborted:
            return

    runner.execute_step(
        '出征准备页面 → ◁ 返回',
        None,  # 返回目标页面因地图层级不同而变化，跳过严格验证
        lambda _: True,
        lambda: bp_page.go_back(),
    )


def _parse_extra_args() -> tuple[int, int]:
    """从 sys.argv 读取 --chapter 和 --map-node，缺省均为 1。"""
    chapter, map_node = 1, 1
    args = sys.argv[1:]
    i = 0
    while i < len(args):
        if args[i] == '--chapter' and i + 1 < len(args):
            chapter = int(args[i + 1])
            i += 2
        elif args[i] == '--map-node' and i + 1 < len(args):
            map_node = int(args[i + 1])
            i += 2
        else:
            i += 1
    return chapter, map_node


def _navigate_to(ctrl, pause: float, *, chapter: int = 1, map_node: int = 1) -> None:
    """从任意已知页面导航到出征准备页面。

    流程: reset → 主页面 → 地图（出征面板） → 章节导航 → 点击地图节点 → 再次点击确认进入
    """
    import time

    from autowsgr.ui.main_page import MainPage
    from autowsgr.ui.map.data import MAP_NODE_POSITIONS, TOTAL_CHAPTERS, MapPanel
    from autowsgr.ui.map.page import MapPage

    if not reset_to_main_page(ctrl, pause):
        return

    # 主页面 → 地图
    screen = ctrl.screenshot()
    if not MainPage.is_current_page(screen):
        return
    MainPage(ctrl).navigate_to(MainPage.Target.SORTIE)
    time.sleep(pause)

    # 确认在地图并切换到出征面板
    screen = ctrl.screenshot()
    if not MapPage.is_current_page(screen):
        return
    if MapPage.get_active_panel(screen) != MapPanel.SORTIE:
        MapPage(ctrl).switch_panel(MapPanel.SORTIE)
        time.sleep(pause)

    # 章节导航 (无 OCR)。先向前滑到第 1 章，再向后到目标章
    map_page = MapPage(ctrl)
    for _ in range(TOTAL_CHAPTERS):
        if not map_page.click_prev_chapter():
            break
        time.sleep(pause * 0.4)
    for _ in range(chapter - 1):
        map_page.click_next_chapter()
        time.sleep(pause * 0.4)
    time.sleep(pause * 0.5)

    # 点击地图节点（两次：第一次选中，第二次确认进入）
    if map_node not in MAP_NODE_POSITIONS:
        info(f'[battle_preparation] map_node={map_node} 不在已知坐标表，尝试节点 1')
        map_node = 1
    pos = MAP_NODE_POSITIONS[map_node]
    ctrl.click(*pos)
    time.sleep(pause * 0.5)
    ctrl.click(*pos)
    time.sleep(pause)


def main() -> None:
    args = parse_e2e_args(
        '出征准备页面 (BattlePreparationPage) e2e 测试',
        precondition='游戏位于出征准备页面 (地图 → 普通关卡 → 选择关卡)',
        default_log_dir='logs/e2e/battle_preparation',
    )
    chapter, map_node = _parse_extra_args()
    ctrl = connect_via_launcher(args.serial, args.log_dir, args.log_level)
    from loguru import logger

    logger.info('=== 出征准备页面 e2e 测试开始 === 章节={} 地图节点={}', chapter, map_node)
    info(f'  章节: {chapter}   地图节点: {map_node}  (可用 --chapter N --map-node M 覆盖)')
    from autowsgr.ui.battle.preparation import BattlePreparationPage

    if not ensure_page(
        ctrl,
        BattlePreparationPage.is_current_page,
        lambda: _navigate_to(ctrl, args.pause, chapter=chapter, map_node=map_node),
        '出征准备页面',
        auto_mode=args.auto,
        pause=args.pause,
    ):
        ctrl.disconnect()
        sys.exit(1)
    runner = UIControllerTestRunner(
        ctrl,
        controller_name='出征准备页面',
        log_dir=args.log_dir,
        auto_mode=args.auto,
        pause=args.pause,
    )
    try:
        run_test(runner)
    except KeyboardInterrupt:
        from testing.ui._framework import warn

        warn('用户中断')
    except Exception as exc:
        from testing.ui._framework import fail

        fail(f'未预期: {exc}')
        logger.opt(exception=True).error('出征准备页面 e2e 测试异常')
    finally:
        runner.finalize()
        runner.print_summary()
        ctrl.disconnect()
        info('设备已断开')

    r = runner.report
    sys.exit(1 if (r.failed > 0 or r.errors > 0) else 0)


if __name__ == '__main__':
    main()



"""游戏重启交互式端到端测试。

直接运行此脚本即可自动连接设备并测试游戏重启流程，适合快速手动验证重启功能。

用法::

    # 自动检测设备
    python testing/ops/startup.py

    # 指定设备 serial
    python testing/ops/startup.py 127.0.0.1:16384

可选参数::

    --log-dir     日志目录 (默认 logs/interactive/restart_game)
    --package     游戏包名 (默认: 官服)

注意: 运行此脚本将会强制关闭并重启游戏，该过程大约需要 30-60 秒。
"""

from __future__ import annotations

import argparse
import io
import sys
import time
from pathlib import Path


# ── UTF-8 输出兼容 (Windows 终端) ──
try:
    if hasattr(sys.stdout, 'reconfigure'):
        sys.stdout.reconfigure(encoding='utf-8', errors='replace')  # type: ignore[union-attr]
        sys.stderr.reconfigure(encoding='utf-8', errors='replace')  # type: ignore[union-attr]
except Exception:
    try:
        if isinstance(sys.stdout, io.TextIOWrapper):
            sys.stdout = io.TextIOWrapper(
                sys.stdout.buffer, encoding='utf-8', errors='replace', line_buffering=True
            )
        if isinstance(sys.stderr, io.TextIOWrapper):
            sys.stderr = io.TextIOWrapper(
                sys.stderr.buffer, encoding='utf-8', errors='replace', line_buffering=True
            )
    except Exception:
        pass

from loguru import logger

from autowsgr.emulator import ADBController
from autowsgr.infra import ConfigManager, setup_logger
from autowsgr.ops.startup import restart_game
from autowsgr.types import GameAPP


def _parse_args() -> argparse.Namespace:
    parser = argparse.ArgumentParser(
        description='游戏重启交互式端到端测试',
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        'serial',
        nargs='?',
        default=None,
        metavar='SERIAL',
        help='ADB serial (留空则自动检测唯一设备)',
    )
    parser.add_argument(
        '--log-dir',
        default=None,
        metavar='DIR',
        help='日志输出目录 (默认: logs/interactive/restart_game)',
    )
    parser.add_argument(
        '--package',
        default=None,
        metavar='PKG',
        help='游戏包名 (默认: 官服)',
    )
    return parser.parse_args()


def main() -> None:
    args = _parse_args()

    # ── 日志 ──
    log_dir = Path(args.log_dir) if args.log_dir else Path('logs/interactive/restart_game')
    cfg = ConfigManager.load()
    channels = cfg.log.effective_channels or None
    setup_logger(log_dir=log_dir, level='DEBUG', save_images=True, channels=channels)

    serial: str | None = args.serial or None

    # 确定包名
    if args.package:
        package = args.package
        logger.info('使用指定包名: {}', package)
    else:
        package = GameAPP.official.package_name
        logger.info('使用官服包名: {}', package)

    logger.info('=' * 50)
    logger.info('游戏重启交互式测试')
    logger.info('  日志: {}', log_dir)
    logger.info('=' * 50)

    # ── 连接设备 ──
    logger.info('正在连接设备{}...', f' ({serial})' if serial else ' (自动检测)')
    ctrl = ADBController(serial=serial or cfg.emulator.serial)
    try:
        dev_info = ctrl.connect()
        logger.info(
            '已连接: {}  分辨率: {}x{}',
            dev_info.serial,
            dev_info.resolution[0],
            dev_info.resolution[1],
        )
    except Exception as exc:
        logger.error('连接设备失败: {}', exc)
        sys.exit(1)

    # ── 执行重启 ──
    try:
        logger.info('开始重启游戏...')
        restart_game(ctrl, package=package)
        logger.info('重启调用完成，等待游戏启动...')

        # 等待游戏启动（约 30-60 秒）
        max_wait = 90
        start_time = time.time()
        screenshoot_ok = False

        while time.time() - start_time < max_wait:
            try:
                screen = ctrl.screenshot()
                if screen is not None and screen.size > 0:
                    screenshoot_ok = True
                    logger.info('游戏已启动，获得有效截图')
                    break
                logger.debug('等待游戏启动...')
            except Exception as e:
                logger.debug('获取截图失败 (预期): {}', e)
            time.sleep(2)

        if not screenshoot_ok:
            logger.warning('等待超时，游戏可能未成功启动')
            sys.exit(1)

        # ── 验证结果 ──
        logger.info('=' * 50)
        logger.info('✓ 游戏重启测试通过')
        logger.info('  - 成功强制停止并冷启动游戏')
        logger.info('  - 游戏已启动且能够获取截图')
        logger.info('=' * 50)

    except Exception as exc:
        logger.error('重启游戏失败: {}', exc)
        logger.error('错误堆栈:', exc_info=True)
        sys.exit(1)


if __name__ == '__main__':
    main()



"""战斗状态枚举与状态转移图。
一次完整的 MAP 类战斗流程::
    PROCEED → FIGHT_CONDITION → SPOT_ENEMY_SUCCESS → FORMATION
        → FIGHT_PERIOD → NIGHT_PROMPT → RESULT → GET_SHIP → PROCEED → ...
"""

from __future__ import annotations

from enum import Enum, auto


class CombatPhase(Enum):
    """战斗阶段。

    每个枚举值代表战斗状态机中的一个离散状态。
    """

    # ── 出征过渡 ──
    START_FIGHT = auto()
    """点击出征后的短暂过渡：检测船坞已满或进入战斗。"""

    # ── 船坞已满 ──
    DOCK_FULL = auto()
    """出征时检测到船坞已满弹窗。"""

    # ── 航行 / 继续 ──
    PROCEED = auto()
    """继续前进 / 回港提示。"""

    # ── 战况选择 ──
    FIGHT_CONDITION = auto()
    """战况选择界面（稳步前进 / 火力万岁 等）。"""

    # ── 索敌 ──
    SPOT_ENEMY_SUCCESS = auto()
    """索敌成功，显示敌方编成。"""

    # ── 阵型选择 ──
    FORMATION = auto()
    """选择阵型界面。"""

    # ── 导弹支援 ──
    MISSILE_ANIMATION = auto()
    """导弹支援动画播放中。"""

    # ── 战斗进行 ──
    FIGHT_PERIOD = auto()
    """昼战 / 夜战战斗动画进行中。"""

    # ── 夜战提示 ──
    NIGHT_PROMPT = auto()
    """夜战选择提示（追击 / 撤退）。"""

    # ── 战果结算 ──
    RESULT = auto()
    """战果评价界面（S/A/B/C/D/SS）。"""

    # ── 掉落 ──
    GET_SHIP = auto()
    """获取舰船掉落。"""

    # ── 旗舰大破 ──
    FLAGSHIP_SEVERE_DAMAGE = auto()
    """旗舰大破强制回港。"""

    # ── 结束页面 ──
    MAP_PAGE = auto()
    """回到地图页面（常规战结束）。"""

    EXERCISE_PAGE = auto()
    """回到演习页面（演习结束）。"""

    EVENT_MAP_PAGE = auto()
    """回到活动地图页面（活动战斗结束）。"""


# ═══════════════════════════════════════════════════════════════════════════════
# 模式分类与状态转移图
# ═══════════════════════════════════════════════════════════════════════════════

PhaseBranch = list[CombatPhase] | dict[str, list[CombatPhase]]
"""分支定义：无条件列表，或按动作名索引的字典。"""


class ModeCategory(Enum):
    """战斗模式大类。

    所有具体 ``CombatMode`` 归入这两类之一，转移图由大类 + ``end_page`` 唯一确定。

    MAP
        多节点地图模式。有战况选择、迂回、导弹支援、舰船掉落、旗舰大破、
        继续前进 / 回港 等完整流程。（常规战、活动战）
    SINGLE
        单点战斗模式。无多节点特性，只有核心战斗循环。
        ``end_page=None`` 时以 RESULT 为终止态。（战役、决战、演习）
    """

    MAP = auto()
    SINGLE = auto()


def build_transitions(
    category: ModeCategory,
    end_page: CombatPhase | None,
) -> dict[CombatPhase, PhaseBranch]:
    """根据模式大类和结束页面自动构建状态转移图。

    Parameters
    ----------
    category:
        ``MAP`` 或 ``SINGLE``。
    end_page:
        战斗结束游戏回到的页面状态。``None`` 表示以 ``RESULT`` 作为终止态。

    Returns
    -------
    dict[CombatPhase, PhaseBranch]
    """
    if category == ModeCategory.MAP:
        return _build_map_transitions(end_page)
    return _build_single_transitions(end_page)


def _build_map_transitions(
    end_page: CombatPhase | None,
) -> dict[CombatPhase, PhaseBranch]:
    """MAP 类：多节点地图战斗的完整转移图。"""
    ep = end_page  # 简写
    t: dict[CombatPhase, PhaseBranch] = {}

    # 地图移动后的着陆节点
    core_nav = [
        CombatPhase.FIGHT_CONDITION,
        CombatPhase.SPOT_ENEMY_SUCCESS,
        CombatPhase.FORMATION,
        CombatPhase.FIGHT_PERIOD,
    ]

    fight_targets = [
        CombatPhase.FORMATION,
        CombatPhase.FIGHT_PERIOD,
        CombatPhase.MISSILE_ANIMATION,
    ]

    # RESULT 之后
    after_result = [CombatPhase.PROCEED]
    if ep is not None:
        after_result.append(ep)
    after_result += [CombatPhase.GET_SHIP, CombatPhase.FLAGSHIP_SEVERE_DAMAGE]

    # ── 各节点 ──
    start = list(core_nav)
    if ep is not None:
        start.append(ep)
    start.append(CombatPhase.DOCK_FULL)
    t[CombatPhase.START_FIGHT] = start

    t[CombatPhase.DOCK_FULL] = []

    proceed_yes = list(core_nav)
    if ep is not None:
        proceed_yes.append(ep)
    t[CombatPhase.PROCEED] = {
        'yes': proceed_yes,
        'no': [ep] if ep is not None else [],
    }

    t[CombatPhase.FIGHT_CONDITION] = [
        CombatPhase.SPOT_ENEMY_SUCCESS,
        CombatPhase.FORMATION,
        CombatPhase.FIGHT_PERIOD,
    ]

    t[CombatPhase.SPOT_ENEMY_SUCCESS] = {
        'fight': list(fight_targets),
        'detour': list(core_nav),
        'retreat': [ep] if ep is not None else [],
    }

    t[CombatPhase.FORMATION] = [
        CombatPhase.FIGHT_PERIOD,
        CombatPhase.MISSILE_ANIMATION,
    ]

    t[CombatPhase.MISSILE_ANIMATION] = [
        CombatPhase.FIGHT_PERIOD,
        CombatPhase.RESULT,
    ]

    t[CombatPhase.FIGHT_PERIOD] = [CombatPhase.NIGHT_PROMPT, CombatPhase.RESULT]
    t[CombatPhase.NIGHT_PROMPT] = {
        'yes': [CombatPhase.RESULT],
        'no': [CombatPhase.RESULT],
    }

    t[CombatPhase.RESULT] = list(after_result)

    # GET_SHIP 后继 = RESULT 后继 去掉 GET_SHIP 自身
    t[CombatPhase.GET_SHIP] = [p for p in after_result if p != CombatPhase.GET_SHIP]

    if ep is not None:
        t[CombatPhase.FLAGSHIP_SEVERE_DAMAGE] = [ep]

    return t


def _build_single_transitions(
    end_page: CombatPhase | None,
) -> dict[CombatPhase, PhaseBranch]:
    """SINGLE 类：单点战斗的精简转移图。"""
    ep = end_page
    t: dict[CombatPhase, PhaseBranch] = {}

    core = [
        CombatPhase.SPOT_ENEMY_SUCCESS,
        CombatPhase.FORMATION,
        CombatPhase.FIGHT_PERIOD,
    ]

    t[CombatPhase.START_FIGHT] = [*list(core), CombatPhase.DOCK_FULL]
    t[CombatPhase.DOCK_FULL] = []

    t[CombatPhase.SPOT_ENEMY_SUCCESS] = {
        'fight': [CombatPhase.FORMATION, CombatPhase.FIGHT_PERIOD],
        'retreat': [ep] if ep is not None else [],
    }

    t[CombatPhase.FORMATION] = [CombatPhase.FIGHT_PERIOD]
    t[CombatPhase.FIGHT_PERIOD] = [CombatPhase.NIGHT_PROMPT, CombatPhase.RESULT]
    t[CombatPhase.NIGHT_PROMPT] = {
        'yes': [CombatPhase.RESULT],
        'no': [CombatPhase.RESULT],
    }

    if ep is not None:
        t[CombatPhase.RESULT] = [ep]
    # ep is None → RESULT 为终止态，无后继

    return t


def resolve_successors(
    transitions: dict[CombatPhase, PhaseBranch],
    phase: CombatPhase,
    last_action: str,
) -> list[CombatPhase]:
    """根据当前状态和上一步动作，解析出候选后继状态列表。

    Parameters
    ----------
    transitions:
        状态转移图。
    phase:
        当前状态。
    last_action:
        上一步动作名称（用于 action-dependent 分支）。

    Returns
    -------
    list[CombatPhase]

    Raises
    ------
    KeyError
        当前状态不在转移图中。
    """
    branch = transitions[phase]

    if isinstance(branch, dict):
        targets = branch.get(last_action)
        if targets is None:
            targets = next(iter(branch.values()))
    else:
        targets = branch

    return list(targets)



"""选船页面 UI 控制器。

已完成，需测试

使用方式::

    from autowsgr.ui.choose_ship_page import ChooseShipPage

    page = ChooseShipPage(ctrl)
    page.click_search_box()
    page.click_first_result()
"""

from __future__ import annotations

import time
from typing import TYPE_CHECKING

from autowsgr.infra.logger import get_logger
from autowsgr.vision import (
    MatchStrategy,
    PixelChecker,
    PixelRule,
    PixelSignature,
)

from .utils import wait_for_page, wait_leave_page
from .utils.ship_list import locate_ship_rows


if TYPE_CHECKING:
    import numpy as np

    from autowsgr.context import GameContext


_log = get_logger('ui')

# ═══════════════════════════════════════════════════════════════════════════════
# 点击坐标 (960x540 基准)
# ═══════════════════════════════════════════════════════════════════════════════

CLICK_SEARCH_BOX: tuple[float, float] = (700 / 960, 30 / 540)
"""搜索框。"""

CLICK_DISMISS_KEYBOARD: tuple[float, float] = (50 / 960, 50 / 540)
"""点击空白区域关闭键盘。"""

CLICK_REMOVE_SHIP: tuple[float, float] = (83 / 960, 167 / 540)
"""「移除」按钮 — 将当前槽位舰船移除。"""

CLICK_FIRST_RESULT: tuple[float, float] = (183 / 960, 167 / 540)
"""搜索结果列表中的第一个结果。"""

#: 选船列表滚动参数
_SCROLL_FROM_Y: float = 0.55
_SCROLL_TO_Y: float = 0.30
_OCR_MAX_ATTEMPTS: int = 3

PAGE_SIGNATURE = PixelSignature(
    name='skill_used',
    strategy=MatchStrategy.ALL,
    rules=[
        PixelRule.of(0.8594, 0.1514, (31, 46, 69), tolerance=30.0),
        PixelRule.of(0.8602, 0.3167, (31, 139, 238), tolerance=30.0),
        PixelRule.of(0.8578, 0.5306, (57, 57, 57), tolerance=30.0),
        PixelRule.of(0.8594, 0.6736, (54, 54, 54), tolerance=30.0),
        PixelRule.of(0.8656, 0.8014, (35, 57, 81), tolerance=30.0),
    ],
)

INPUT_SIGNATURE = PixelSignature(
    name='choose_ship_input',
    strategy=MatchStrategy.ALL,
    rules=[
        PixelRule.of(0.3109, 0.9417, (253, 253, 253), tolerance=30.0),
        PixelRule.of(0.4437, 0.9417, (253, 253, 253), tolerance=30.0),
        PixelRule.of(0.5883, 0.9347, (253, 253, 253), tolerance=30.0),
    ],
)

# ═══════════════════════════════════════════════════════════════════════════════
# 页面控制器
# ═══════════════════════════════════════════════════════════════════════════════


class ChooseShipPage:
    """选船页面控制器。

    从出征准备页面点击舰船槽位后进入此页面。
    提供搜索、选择、移除舰船等原子操作。

    Parameters
    ----------
    ctrl:
        Android 设备控制器实例。
    """

    def __init__(self, ctx: GameContext) -> None:
        self._ctx = ctx
        self._ctrl = ctx.ctrl

    # ── 页面识别 ──────────────────────────────────────────────────────────

    @staticmethod
    def is_current_page(screen: np.ndarray) -> bool:
        """判断截图是否为选船页面。

        .. warning::
            尚未实现像素签名采集，当前始终返回 False。
            选船页面识别由 ops 层通过图像模板匹配完成。

        Parameters
        ----------
        screen:
            截图 (HxWx3, RGB)。
        """
        result = PixelChecker.check_signature(screen, PAGE_SIGNATURE)
        return result.matched

    def _wait_leave_current_page(self, timeout: float = 5.0):
        wait_leave_page(self._ctrl, self.is_current_page, timeout=timeout)

    # ── 操作 ──────────────────────────────────────────────────────────────
    def ensure_search_box(self) -> None:
        """点击搜索框，准备输入舰船名。"""
        _log.info('[UI] 选船 → 打开搜索框')
        self._ctrl.click(*CLICK_SEARCH_BOX)
        wait_for_page(
            self._ctrl,
            lambda screen: PixelChecker.check_signature(screen, INPUT_SIGNATURE).matched,
            timeout=5.0,
        )

    def input_ship_name(self, name: str) -> None:
        """在搜索框中输入舰船名。

        调用前应先 :meth:`click_search_box`。

        Parameters
        ----------
        name:
            舰船名 (中文)。
        """
        _log.debug("[UI] 选船 → 输入舰船名 '{}'", name)
        self._ctrl.text(name)

    def ensure_dismiss_keyboard(self) -> None:
        """点击空白区域关闭软键盘。"""
        _log.debug('[UI] 选船 → 关闭键盘')
        self._ctrl.click(*CLICK_DISMISS_KEYBOARD)
        wait_leave_page(
            self._ctrl,
            lambda screen: PixelChecker.check_signature(screen, INPUT_SIGNATURE).matched,
            timeout=5.0,
        )

    def click_first_result(self) -> None:
        """点击搜索结果中的第一个舰船。"""
        _log.debug('[UI] 选船 → 点击第一个结果')
        self._ctrl.click(*CLICK_FIRST_RESULT)

    def click_remove(self) -> None:
        """点击「移除」按钮，移除当前槽位的舰船。"""
        _log.debug('[UI] 选船 → 移除舰船')
        self._ctrl.click(*CLICK_REMOVE_SHIP)

    def change_single_ship(
        self,
        name: str | None,
        *,
        use_search: bool = True,
    ) -> None:
        """更换/移除当前槽位的舰船。

        使用 DLL 行定位 + OCR 在选船列表中查找目标舰船并点击。
        最多重试 ``_OCR_MAX_ATTEMPTS`` 次, 每次失败后向上滚动列表。

        Parameters
        ----------
        name:
            目标舰船名; ``None`` 表示移除当前槽位舰船。
        use_search:
            是否使用搜索框输入舰船名来过滤列表。
            常规出征为 ``True`` (默认), 决战为 ``False``
            (决战选船界面没有搜索框)。
        """
        if name is None:
            self.click_remove()
            self._wait_leave_current_page()
            return

        if self._ctx.ocr is None:
            _log.warning('[UI] 未提供 OCR 引擎, 无法识别选船列表')
            return
        if use_search:
            self.ensure_search_box()
            self.input_ship_name(name)
            self.ensure_dismiss_keyboard()
        found = self._click_ship_in_list(name)
        if not found:
            _log.error("[UI] 未在选船列表中找到 '{}'", name)
            raise RuntimeError(f"未找到目标舰船 '{name}'")

        self._wait_leave_current_page()

    def _click_ship_in_list(self, name: str) -> bool:
        """在选船列表页使用 DLL 定位 + OCR 识别舰船名并点击目标。

        最多重试 ``_OCR_MAX_ATTEMPTS`` 次, 每次失败后向上滚动列表。

        Parameters
        ----------
        name:
            目标舰船名 (精确名称)。

        Returns
        -------
        bool
            ``True`` 表示已成功点击目标; ``False`` 表示全程未找到。
        """
        assert self._ctx.ocr is not None

        for attempt in range(_OCR_MAX_ATTEMPTS):
            screen = self._ctrl.screenshot()
            hits = locate_ship_rows(self._ctx.ocr, screen)

            for matched, cx, cy in hits:
                if matched != name:
                    continue
                _log.info(
                    "[UI] 选船 DLL+OCR -> '{}' (第 {}/{} 次), 点击 ({:.3f}, {:.3f})",
                    name,
                    attempt + 1,
                    _OCR_MAX_ATTEMPTS,
                    cx,
                    cy,
                )
                time.sleep(1.0)
                self._ctrl.click(cx, cy)
                return True

            _log.warning(
                "[UI] 选船列表未匹配到 '{}' (第 {}/{} 次), 向上滚动",
                name,
                attempt + 1,
                _OCR_MAX_ATTEMPTS,
            )
            if attempt < _OCR_MAX_ATTEMPTS - 1:
                self._ctrl.swipe(0.4, _SCROLL_FROM_Y, 0.4, _SCROLL_TO_Y, duration=0.4)
                time.sleep(0.5)

        return False



"""UI 控制层 — 页面识别与交互操作。

每个游戏页面对应一个控制器类，封装：

1. **页面识别** — 通过像素特征检测当前是否在该页面
2. **状态查询** — 读取页面内动态状态（选中标签、开关等）
3. **操作动作** — 点击按钮、切换标签等

状态查询为 ``staticmethod``，只需截图数组即可调用；
操作动作需要 :class:`~autowsgr.emulator.controller.AndroidController` 实例。

导航操作 (``go_back``、``navigate_to`` 等) 内置截图验证，
点击后反复截图确认页面已切换，超时抛出 :class:`NavigationError`。
所有导航均使用 **正向验证** (目标页面签名匹配)，不再使用离开判定。

浮层处理:
    导航过程中自动检测并消除游戏浮层 (新闻公告、每日签到)。
    浮层模块: :mod:`autowsgr.ui.overlay`

自动导航 / 兜底回主页:
    ``goto_page()``、``go_main_page()`` 等跨页面路由属于 **游戏层**
    (GameOps) 的职责，不在 UI 控制层中实现。
    UI 层只提供：页面识别、导航验证、单步操作。

页面导航树::

    主页面 (MainPage)
    ├── 地图页面 (MapPage)                  ← 出征
    │   ├── [面板] 出征/演习/远征/战役/决战
    │   └── 出征准备 (BattlePreparationPage)
    │       └── → 浴室 (BathPage)           ← 跨级快捷通道
    ├── 任务页面 (MissionPage)              ← 任务
    ├── 后院页面 (BackyardPage)             ← 主页图标
    │   ├── 浴室 (BathPage)
    │   │   └── 选择修理 (ChooseRepairPage)
    │   └── 食堂 (CanteenPage)
    └── 侧边栏 (SidebarPage)               ← ≡ 按钮
        ├── 建造 (BuildPage)
        │   └── [标签] 建造/解体/开发/废弃
        ├── 强化 (IntensifyPage)
        │   └── [标签] 强化/改修/技能
        └── 好友 (FriendPage)

页面识别注册中心::

    from autowsgr.ui import get_current_page

    screen = ctrl.screenshot()
    page_name = get_current_page(screen)  # "主页面" / "地图页面" / ...

导航路径查找::

    from autowsgr.ui.navigation import find_path
    from autowsgr.types import PageName

    path = find_path(PageName.MAIN, PageName.BUILD)
    for edge in path:
        edge.action(ctrl)

使用方式::

    from autowsgr.ui import BattlePreparationPage, Panel

    page = BattlePreparationPage(ctrl)
    screen = ctrl.screenshot()
    if BattlePreparationPage.is_current_page(screen):
        fleet = BattlePreparationPage.get_selected_fleet(screen)
        page.start_battle()
"""

# ── 控制器 ─────────────────────────────────────────────────────────────
from autowsgr.types import PageName
from autowsgr.ui.main_page.constants import OverlayKind
from autowsgr.ui.page import (
    get_current_page,
    get_registered_pages,
    register_page,
)

# ── 标签页统一检测层 ──────────────────────────────────────────────
from autowsgr.ui.tabbed_page import (
    TabbedPageType,
)
from autowsgr.ui.utils import (
    NavConfig,
    NavigationError,
    click_and_wait_for_page,
    wait_for_page,
    wait_leave_page,
)

from .backyard_page import BackyardPage, BackyardTarget
from .bath_page import BathPage
from .battle import BattlePreparationPage, Panel, RepairStrategy
from .build_page import BuildPage, BuildTab
from .canteen_page import CanteenPage
from .choose_ship_page import ChooseShipPage
from .decisive import DecisiveBattlePage, DecisiveMapController
from .event.event_page import BaseEventPage
from .friend_page import FriendPage
from .intensify_page import IntensifyPage, IntensifyTab
from .main_page import MainPage
from .map.data import MAP_DATABASE, MapIdentity, MapPanel
from .map.page import MapPage
from .mission_page import MissionInfo, MissionPage, MissionPanel
from .sidebar_page import SidebarPage, SidebarTarget


# ── 兼容别名 ─────────────────────────────────────────────────────
MainPageTarget = MainPage.Target
OverlayType = OverlayKind

# ── 注册所有页面识别器 ──


register_page(PageName.MAIN, MainPage.is_current_page)
register_page(PageName.MAP, MapPage.is_current_page)
register_page(PageName.BATTLE_PREP, BattlePreparationPage.is_current_page)
register_page(PageName.SIDEBAR, SidebarPage.is_current_page)
register_page(PageName.MISSION, MissionPage.is_current_page)
register_page(PageName.BACKYARD, BackyardPage.is_current_page)
register_page(PageName.BATH, BathPage.is_current_page)
register_page(PageName.CANTEEN, CanteenPage.is_current_page)
register_page(PageName.BUILD, BuildPage.is_current_page)
register_page(PageName.INTENSIFY, IntensifyPage.is_current_page)
register_page(PageName.FRIEND, FriendPage.is_current_page)
register_page(PageName.DECISIVE_BATTLE, DecisiveBattlePage.is_current_page)
register_page(PageName.EVENT_MAP, BaseEventPage.is_current_page)

__all__ = [
    # ── 数据 ──
    'MAP_DATABASE',
    # ── 控制器 ──
    'BackyardPage',
    'BackyardTarget',
    'BaseEventPage',
    'BathPage',
    'BattlePreparationPage',
    'BuildPage',
    'BuildTab',
    'CanteenPage',
    'ChooseShipPage',
    'DecisiveBattlePage',
    'DecisiveMapController',
    'FriendPage',
    'IntensifyPage',
    'IntensifyTab',
    'MainPage',
    'MainPageTarget',
    'MapIdentity',
    'MapPage',
    'MapPanel',
    'MissionInfo',
    'MissionPage',
    'MissionPanel',
    # ── 导航基础设施 ──
    'NavConfig',
    'NavigationError',
    # ── 浮层 ──
    'OverlayKind',
    'OverlayType',
    'Panel',
    'RepairStrategy',
    'SidebarPage',
    'SidebarTarget',
    # ── 标签页统一检测 ──
    'TabbedPageType',
    'click_and_wait_for_page',
    'get_current_page',
    'get_registered_pages',
    'register_page',
    'wait_for_page',
    'wait_leave_page',
]



"""模拟器层 — 设备控制与进程管理。

提供两大核心能力：

1. **设备控制** (`AndroidController` / `ScrcpyController`)：
   截图、点击、滑动、按键、应用管理等纯设备操作。
   所有触控坐标使用相对值 (0.0-1.0)。

2. **进程管理** (`EmulatorProcessManager` / `create_emulator_manager`)：
   在宿主操作系统上启动、停止、检测模拟器进程。
"""

from .controller import (
    AndroidController,
    DeviceInfo,
    ScrcpyController,
)
from .detector import (
    EmulatorCandidate,
    detect_emulators,
    prompt_user_select,
    resolve_serial,
)
from .os_control import (
    EmulatorProcessManager,
    LinuxEmulatorManager,
    MacEmulatorManager,
    WindowsEmulatorManager,
    create_emulator_manager,
)


__all__ = [
    'AndroidController',
    'DeviceInfo',
    'EmulatorCandidate',
    'EmulatorProcessManager',
    'LinuxEmulatorManager',
    'MacEmulatorManager',
    'ScrcpyController',
    'WindowsEmulatorManager',
    'create_emulator_manager',
    'detect_emulators',
    'prompt_user_select',
    'resolve_serial',
]



"""活动导航 — 主页面 → 活动地图的专用导航流程。

逻辑: 清除浮层 → 等待活动图标模板出现 → 点击并等待活动页面。
"""

from __future__ import annotations

import time
from typing import TYPE_CHECKING

from autowsgr.infra.logger import get_logger
from autowsgr.vision import ImageChecker

from .constants import NavCoord, Sig, Target
from .overlays import detect_overlay, dismiss_overlay


if TYPE_CHECKING:
    from collections.abc import Callable

    import numpy as np

    from autowsgr.emulator import AndroidController
    from autowsgr.vision import ImageTemplate

_log = get_logger('ui')

# ── 活动图标模板 (延迟加载) ──────────────────────────────────────────────

_event_icon_templates: list[ImageTemplate] | None = None


def _get_event_icon_templates() -> list[ImageTemplate]:
    """延迟加载活动入口图标模板。"""
    global _event_icon_templates
    if _event_icon_templates is None:
        from autowsgr.image_resources._lazy import load_template

        _event_icon_templates = [
            load_template(
                'event/event_icon_20260212_720p.png',
                name='event_icon',
                source_resolution=(1280, 720),
            ),
        ]
    return _event_icon_templates


# ── 内部: 单次导航尝试 ──────────────────────────────────────────────────


def _try_navigate_to_event(
    ctrl: AndroidController,
    event_checker: Callable[[np.ndarray], bool],
    *,
    icon_wait: float = 3.0,
    nav_wait: float = 5.0,
) -> bool:
    """执行一次活动导航尝试。

    流程:
        1. 确保无浮层覆盖
        2. 等待活动图标模板出现 (图像模板匹配)
        3. 点击图标并等待活动页面签名

    Returns
    -------
    bool
        ``True`` — 成功到达活动页面；``False`` — 本次失败。
    """
    from autowsgr.vision import PixelChecker

    # ① 清除浮层
    screen = ctrl.screenshot()
    overlay = detect_overlay(screen)
    if overlay is not None:
        _log.debug('[UI] 活动导航: 先消除浮层 {}', overlay.value)
        dismiss_overlay(ctrl, overlay)
        time.sleep(0.5)
        screen = ctrl.screenshot()

    # 确认仍在主页面
    if not PixelChecker.check_signature(screen, Sig.PAGE.ps).matched:
        _log.warning('[UI] 活动导航: 当前不在主页面基础态')
        return False

    # ② 等待活动图标模板出现
    templates = _get_event_icon_templates()
    deadline = time.time() + icon_wait
    detail = None
    while time.time() < deadline:
        screen = ctrl.screenshot()
        detail = ImageChecker.find_any(screen, templates, confidence=0.8)
        if detail is not None:
            break
        time.sleep(0.3)

    if detail is None:
        _log.warning('[UI] 活动导航: 等待 {:.1f}s 后仍未检测到活动图标', icon_wait)
        return False

    # ③ 点击活动图标并等待活动页面
    coord = NavCoord.EVENT.xy
    _log.debug('[UI] 主页面 → 活动')
    ctrl.click(*coord)
    time.sleep(1.0)

    deadline = time.time() + nav_wait
    while time.time() < deadline:
        screen = ctrl.screenshot()
        if event_checker(screen):
            _log.info('[UI] 已到达活动地图页面')
            return True
        # 点击后可能触发的浮层 (如预约页) 由 overlay 机制统一处理
        overlay = detect_overlay(screen)
        if overlay is not None:
            _log.debug('[UI] 活动导航: 点击后出现浮层 {}, 消除', overlay.value)
            dismiss_overlay(ctrl, overlay)
        time.sleep(0.5)

    _log.warning('[UI] 活动导航: 点击后 {:.1f}s 内未到达活动页面', nav_wait)
    return False


# ── 公开: 带重试的导航入口 ──────────────────────────────────────────────


def navigate_to_event(
    ctrl: AndroidController,
    *,
    is_base_page: type,  # MainPage class (保留兼容签名)
    max_retries: int = 3,
) -> None:
    """导航到活动地图 — 带自动重试。

    Raises
    ------
    NavigationError
        超过最大重试次数仍未到达活动地图页面。
    """
    from autowsgr.ui.utils import NavigationError

    from .controller import _get_target_checker

    event_checker = _get_target_checker(Target.EVENT)

    for attempt in range(1, max_retries + 1):
        _log.debug('[UI] 活动导航: 尝试 {}/{}', attempt, max_retries)
        if _try_navigate_to_event(ctrl, event_checker):
            return
        time.sleep(1.0)

    raise NavigationError(
        f'活动导航失败: {max_retries} 次尝试后仍未进入活动地图页面',
        screen=ctrl.screenshot(),
    )



"""浴室页面 UI 模块。

公开 API::

    from autowsgr.ui.bath_page import BathPage, RepairShipInfo
"""

from .page import (
    BathPage,
    RepairShipInfo,
)
from .recognition import recognize_repair_cards
from .signatures import (
    CHOOSE_REPAIR_OVERLAY_SIGNATURE,
    PAGE_SIGNATURE,
)


__all__ = [
    'CHOOSE_REPAIR_OVERLAY_SIGNATURE',
    'PAGE_SIGNATURE',
    'BathPage',
    'RepairShipInfo',
    'recognize_repair_cards',
]



"""出征准备 — 舰船状态检测与舰队信息识别。

提供血量检测、等级 OCR 识别等静态 / 实例方法。
"""

from __future__ import annotations

import re
import time
from dataclasses import dataclass, field
from typing import TYPE_CHECKING

import cv2

from autowsgr.infra.logger import get_logger
from autowsgr.types import ShipDamageState
from autowsgr.ui.battle.base import BaseBattlePreparation
from autowsgr.vision import PixelChecker

from .blood import classify_blood
from .constants import (
    BLOOD_BAR_PROBE,
    SHIP_LEVEL_CROP,
)


if TYPE_CHECKING:
    import numpy as np

    from autowsgr.context.ship import Ship


_log = get_logger('ui.preparation')


# ═══════════════════════════════════════════════════════════════════════════════
# 数据类
# ═══════════════════════════════════════════════════════════════════════════════


@dataclass
class FleetInfo:
    """舰队详细信息。"""

    fleet_id: int | None = None
    """舰队编号 (1-4)，``None`` 表示未指定。"""
    ship_levels: dict[int, int | None] = field(default_factory=dict)
    """槽位号 (0-5) → 等级，无法识别或无舰船则为 ``None``。"""
    ship_damage: dict[int, ShipDamageState] = field(default_factory=dict)
    """槽位号 (0-5) → 血量状态。"""

    def to_ships(self, names: list[str | None] | None = None) -> list[Ship]:
        """将舰队信息转换为 Ship 列表。

        Parameters
        ----------
        names:
            可选的舰船名称列表 (0-indexed, 与槽位对应)。
            ``None`` 元素或缺少的索引将使用空字符串。

        Returns
        -------
        list[Ship]
            按槽位顺序排列，跳过无舰船的槽位。
        """
        from autowsgr.context.ship import Ship

        ships: list[Ship] = []
        for i in range(6):
            damage = self.ship_damage.get(i, ShipDamageState.NORMAL)
            if damage == ShipDamageState.NO_SHIP:
                continue
            name = ''
            if names and i < len(names) and names[i] is not None:
                name = names[i]
            ships.append(
                Ship(
                    name=name,
                    level=self.ship_levels.get(i) or 0,
                    damage_state=damage,
                )
            )
        return ships


# ═══════════════════════════════════════════════════════════════════════════════
# Mixin
# ═══════════════════════════════════════════════════════════════════════════════


class DetectionMixin(BaseBattlePreparation):
    """舰船状态检测 Mixin。

    依赖 :class:`~autowsgr.ui.battle.base.BaseBattlePreparation` 提供的
    ``_ctrl``, ``_ocr``, ``get_selected_fleet``, ``select_fleet``。
    """

    # ── 血量检测 ──────────────────────────────────────────────────────────

    @staticmethod
    def detect_ship_damage(screen: np.ndarray) -> dict[int, ShipDamageState]:
        """检测 6 个舰船槽位的血量状态。

        Returns
        -------
        dict[int, ShipDamageState]
            槽位号 (0-5) → 血量状态。
        """
        result: dict[int, ShipDamageState] = {}
        for slot, (x, y) in BLOOD_BAR_PROBE.items():
            pixel = PixelChecker.get_pixel(screen, x, y)
            result[slot] = classify_blood(pixel)
        _log.debug(
            '[准备页] 血量检测: {}',
            ' | '.join(f'槽{i}={result[i].name}' for i in range(len(result))),
        )
        return result

    # ── 等级 OCR ──────────────────────────────────────────────────────────

    def _recognize_fleet_levels(
        self,
        screen: np.ndarray,
    ) -> dict[int, int | None]:
        """从准备页截图中 OCR 识别每艘舰船的等级。

        读取各舰船卡片上的 ``Lv.XX`` / ``Lv XX`` 文本，提取数字部分。

        Parameters
        ----------
        screen:
            出征准备页的截图。

        Returns
        -------
        dict[int, int | None]
            槽位号 (0-5) → 等级。无法识别或无舰船则为 ``None``。
        """
        levels: dict[int, int | None] = {}
        ocr = self._ocr
        if ocr is None:
            _log.warning('[UI] 未提供 OCR 引擎，无法识别舰船等级')
            return dict.fromkeys(range(6))

        # 先检测哪些槽位有舰船
        damage = self.detect_ship_damage(screen)

        for slot in range(6):
            if damage.get(slot) == ShipDamageState.NO_SHIP:
                levels[slot] = None
                continue

            crop_region = SHIP_LEVEL_CROP.get(slot)
            if crop_region is None:
                levels[slot] = None
                continue

            cropped = PixelChecker.crop(screen, *crop_region)
            # 4x 上采样提升小字 OCR 准确率 (对齐 legacy check_level)
            upscaled = cv2.resize(
                cropped,
                (cropped.shape[1] * 4, cropped.shape[0] * 4),
            )

            # 用多结果模式避免 recognize_single 选中噪声文本
            level = self._best_level_from_results(
                ocr.recognize(upscaled, allowlist='0123456789Llv.V'),
            )
            levels[slot] = level

            if level is not None:
                _log.debug('[UI] 槽位{} 等级: Lv.{}', slot, level)
            else:
                _log.debug('[UI] 槽位{} 等级识别失败', slot)

        _log.info(
            '[准备页] 等级检测: {}',
            ' | '.join(
                f'槽{i}={"Lv." + str(levels[i]) if levels[i] is not None else "无"}'
                for i in range(6)
            ),
        )
        return levels

    # ── 舰队信息聚合 ─────────────────────────────────────────────────────

    def detect_fleet_info(
        self,
        fleet_id: int | None = None,
    ) -> FleetInfo:
        """识别指定舰队的详细信息（等级、血量）。

        若 ``fleet_id`` 不为 ``None`` 且与当前选中的舰队不同，
        将先切换到目标舰队。

        Parameters
        ----------
        fleet_id:
            目标舰队编号 (1-4)。为 ``None`` 则不切换舰队。

        Returns
        -------
        FleetInfo
            包含舰队编号、各舰船等级和血量信息。
        """
        if fleet_id is not None:
            screen = self._ctrl.screenshot()
            current_fleet = self.get_selected_fleet(screen)
            if current_fleet != fleet_id:
                self.select_fleet(fleet_id)
                time.sleep(0.5)

        screen = self._ctrl.screenshot()
        actual_fleet = self.get_selected_fleet(screen)
        damage = self.detect_ship_damage(screen)
        levels = self._recognize_fleet_levels(screen)

        info = FleetInfo(
            fleet_id=actual_fleet,
            ship_levels=levels,
            ship_damage=damage,
        )

        _log.info(
            '[UI] 舰队 {} 信息: {}',
            actual_fleet or '?',
            ' | '.join(
                f'槽{i}=Lv.{levels.get(i, "?")} {damage[i].name if i in damage else "?"}'
                for i in range(6)
            ),
        )
        return info

    # ── 工具 ──────────────────────────────────────────────────────────────

    @staticmethod
    def _parse_level(text: str) -> int | None:
        """解析等级文本。

        支持格式: ``"Lv.120"``, ``"Lv120"``, ``"lv 98"``, ``"120"`` 等。
        OCR 常见噪声: ``"0.106"`` (L 误识为 0), ``"1V.31"`` (前缀数字),
        ``"497"`` (星级数字粘连) 等。
        """
        # 1) 尝试匹配 [L10I]V.XX 模式 (L 可被 OCR 误识为 1/0/I)
        m = re.search(r'(?i)[l10i]\s*v\.?\s*(\d{1,3})', text)
        if m:
            val = int(m.group(1))
            if 1 <= val <= 200:
                return val

        # 2) 尝试匹配 V.XX 模式 (缺失 L)
        m = re.search(r'(?i)v\.?\s*(\d{1,3})', text)
        if m:
            val = int(m.group(1))
            if 1 <= val <= 200:
                return val

        # 3) 回退: 取最后一个合法数字 (跳过星级等前缀噪声)
        for m in reversed(list(re.finditer(r'\d+', text))):
            val = int(m.group())
            if 1 <= val <= 200:
                return val
            # 3 位以上数字 > 200 时尝试去掉首位 (星级粘连)
            s = m.group()
            if val > 200 and len(s) >= 3:
                val2 = int(s[1:])
                if 1 <= val2 <= 200:
                    return val2

        return None

    @classmethod
    def _best_level_from_results(cls, results: list) -> int | None:
        """从多个 OCR 结果中选取最佳等级值。

        优先选择包含 LV/V 模式的结果 (更可能是等级文本),
        其次选择纯数字回退结果，避免噪声文本干扰。
        """
        # 按优先级分桶: lv_match > fallback
        lv_candidates: list[int] = []
        fallback_candidates: list[int] = []

        for r in results:
            text = r.text.strip()
            if not text:
                continue

            # 有 V 字母 → 大概率是 LV.XX
            if re.search(r'(?i)v', text):
                val = cls._parse_level(text)
                if val is not None:
                    lv_candidates.append(val)
            else:
                val = cls._parse_level(text)
                if val is not None:
                    fallback_candidates.append(val)

        if lv_candidates:
            return lv_candidates[0]
        if fallback_candidates:
            return fallback_candidates[0]
        return None



"""战役面板 Mixin — 难度选择与进入战役出征准备。"""

from __future__ import annotations

import time
from dataclasses import dataclass

from autowsgr.infra.logger import get_logger
from autowsgr.types import PageName
from autowsgr.ui.map.base import BaseMapPage
from autowsgr.ui.map.data import (
    CAMPAIGN_POSITIONS,
    CLICK_DIFFICULTY,
    DIFFICULTY_EASY_COLOR,
    DIFFICULTY_HARD_COLOR,
    MapPanel,
)
from autowsgr.ui.map.panels.sortie import (
    LOOT_MAX,
    SHIP_MAX,
    recognize_loot_count,
    recognize_ship_count,
)
from autowsgr.ui.utils import wait_for_page
from autowsgr.vision import PixelChecker


_log = get_logger('ui')


class CampaignPanelMixin(BaseMapPage):
    """Mixin: 战役面板操作 — 难度识别 / 切换 / 进入战役。"""

    # ── 查询 ─────────────────────────────────────────────────────────────

    def recognize_difficulty(self) -> str | None:
        """通过检测难度按钮颜色识别当前难度。

        切换图标为蓝色时，说明可以切换为 easy，当前为 hard；反之亦然。

        Returns
        -------
        str | None
            ``"easy"`` / ``"hard"``，识别失败返回 ``None``。

        Raises
        ------
        RuntimeError
            超过重试次数仍无法识别。
        """
        retry = 0
        while retry < 10:
            screen = self._ctrl.screenshot()
            px = PixelChecker.get_pixel(screen, *CLICK_DIFFICULTY)
            if px.near(DIFFICULTY_EASY_COLOR, tolerance=50):
                return 'hard'
            elif px.near(DIFFICULTY_HARD_COLOR, tolerance=50):
                return 'easy'
            time.sleep(0.25)
            retry += 1

        _log.warning('[UI] 无法识别难度: 检测点颜色不匹配简单或困难')
        raise RuntimeError('无法识别战役难度')

    # ── 操作 ─────────────────────────────────────────────────────────────

    def enter_campaign(
        self,
        map_index: int = 2,
        difficulty: str = 'hard',
        campaign_name: str = '未知',
    ) -> None:
        """进入战役: 选择难度和战役类型，直接到达出征准备页面。

        Parameters
        ----------
        map_index:
            战役编号 (1-5: 航母/潜艇/驱逐/巡洋/战列)。
        difficulty:
            难度 ``"easy"`` 或 ``"hard"``。
        campaign_name:
            战役名称 (仅用于日志)。

        Raises
        ------
        ValueError
            战役编号或难度无效。
        NavigationError
            导航超时。
        """
        from autowsgr.ui.battle.preparation import BattlePreparationPage

        if map_index not in CAMPAIGN_POSITIONS:
            raise ValueError(f'战役编号必须为 1-5，收到: {map_index}')
        if difficulty not in ('easy', 'hard'):
            raise ValueError(f'难度必须为 easy 或 hard，收到: {difficulty}')

        _log.info(
            '[UI] 地图页面 → 进入战役 {} ({})',
            campaign_name,
            difficulty,
        )

        # 1. 切换到战役面板
        self.ensure_panel(MapPanel.BATTLE)

        # 2. 选择难度
        if self.recognize_difficulty() != difficulty:
            self._ctrl.click(*CLICK_DIFFICULTY)
        while self.recognize_difficulty() != difficulty:
            _log.debug('[UI] 等待难度切换到 {}…', difficulty)
            time.sleep(0.25)
        time.sleep(0.75)

        # 3. 选择战役
        self._ctrl.click(*CAMPAIGN_POSITIONS[map_index])
        time.sleep(0.5)

        # 等待到达出征准备
        wait_for_page(
            self._ctrl,
            checker=BattlePreparationPage.is_current_page,
            source=f'地图-战役 {campaign_name}',
            target=PageName.BATTLE_PREP,
        )

    # ── 今日获取数量 ──────────────────────────────────────────────────────

    @dataclass
    class AcquisitionCounts:
        """今日获取数量。"""

        ship_count: int | None = None
        """今日已获取舰船数量。"""
        ship_max: int = SHIP_MAX
        """今日舰船获取上限。"""
        loot_count: int | None = None
        """今日已获取战利品数量。"""
        loot_max: int = LOOT_MAX
        """今日战利品获取上限。"""

    def _recognize_acquisition_counts(
        self,
        screen,
    ) -> AcquisitionCounts:
        """从截图中 OCR 识别今日舰船与战利品获取数量。

        Parameters
        ----------
        screen:
            当前截图 (numpy 数组)。

        Returns
        -------
        AcquisitionCounts
            识别到的数量信息, 无法识别的字段为 ``None``。
        """
        result = self.AcquisitionCounts()
        ocr = self._ocr
        if ocr is None:
            _log.warning('[UI] 未提供 OCR 引擎，无法识别获取数量')
            return result

        result.loot_count = recognize_loot_count(screen, ocr)
        result.ship_count = recognize_ship_count(screen, ocr)
        return result

    def get_acquisition_counts(self) -> AcquisitionCounts:
        """获取今日舰船与战利品获取数量。

        自动确保当前位于出征面板，然后截图并执行 OCR 识别。

        Returns
        -------
        AcquisitionCounts
            识别到的数量信息。
        """
        self.ensure_panel(MapPanel.SORTIE)
        time.sleep(0.5)
        screen = self._ctrl.screenshot()
        return self._recognize_acquisition_counts(screen)



"""决战过程控制器（状态机核心）。

``DecisiveController`` 是决战的最高层编排器，**不直接执行任何 UI 操作**，
而是通过以下下层控制器的接口组装完整流程：

- :class:`~autowsgr.ui.decisive.map_controller.DecisiveMapController`
  — 决战地图页所有交互（overlay 处理、出征、撤退、节点间修理）
- :class:`~autowsgr.ui.decisive.preparation.DecisiveBattlePreparationPage`
  — 出征准备页操作（编队、修理、开始战斗）
- :class:`~autowsgr.ui.decisive.battle_page.DecisiveBattlePage`
  — 决战总览页操作（章节导航、重置）

继承体系::

    DecisiveBase                           ← base.py   (成员声明 & 初始化)
    ├── DecisiveChapterOps(DecisiveBase)   ← chapter.py (章节管理)
    ├── DecisivePhaseHandlers(DecisiveBase)← handlers.py (阶段处理器)
    └── DecisiveController(                ← 本文件      (状态机编排)
            DecisivePhaseHandlers,
            DecisiveChapterOps,
        )
"""

from __future__ import annotations

import enum

from autowsgr.infra.logger import get_logger
from autowsgr.ops.decisive.chapter import DecisiveChapterOps
from autowsgr.ops.decisive.handlers import DecisivePhaseHandlers
from autowsgr.types import DecisivePhase


_log = get_logger('ops.decisive')

# ─────────────────────────────────────────────────────────────────────────────
# 结果枚举
# ─────────────────────────────────────────────────────────────────────────────


class DecisiveResult(enum.Enum):
    """决战单轮的最终结局。"""

    CHAPTER_CLEAR = 'chapter_clear'
    """大关通关 (3 个小关全部完成)。"""

    RETREAT = 'retreat'
    """主动撤退 (清空进度)。"""

    LEAVE = 'leave'
    """暂离保存 (保留进度退出)。"""

    ERROR = 'error'
    """异常退出。"""


# ─────────────────────────────────────────────────────────────────────────────
# 组装控制器
# ─────────────────────────────────────────────────────────────────────────────


class DecisiveController(DecisivePhaseHandlers, DecisiveChapterOps):
    """决战过程控制器（状态机核心）。

    通过多重继承组合:
    - :class:`DecisivePhaseHandlers` — 所有阶段处理器
    - :class:`DecisiveChapterOps` — 章节管理操作

    两者均继承 :class:`~autowsgr.ops.decisive.base.DecisiveBase`，
    成员初始化由 ``DecisiveBase.__init__`` 统一完成。
    """

    # ── 主入口 ────────────────────────────────────────────────────────────

    def run(self) -> DecisiveResult:
        """执行一轮完整决战（3 个小关）。"""
        _log.info('[决战] 开始第 {} 章决战', self._config.chapter)
        self._state.reset()
        self._resume_mode = False
        self._has_chosen_fleet = False
        self._prepare_entry_state()
        self._state.phase = DecisivePhase.ENTER_MAP
        try:
            return self._main_loop()
        except Exception:
            _log.exception('[决战] 执行异常')
            self._state.phase = DecisivePhase.FINISHED
            return DecisiveResult.ERROR

    def run_for_times(self, times: int = 1) -> list[DecisiveResult]:
        """执行多轮决战；遇到 LEAVE / ERROR 时提前停止。"""
        results: list[DecisiveResult] = []
        for i in range(times):
            _log.info('[决战] 第 {}/{} 轮', i + 1, times)
            result = self.run()
            results.append(result)
            if result in (DecisiveResult.LEAVE, DecisiveResult.ERROR):
                _log.warning('[决战] 第 {} 轮终止: {}', i + 1, result.value)
                break
        return results

    # ── 主循环 ────────────────────────────────────────────────────────────

    def _main_loop(self) -> DecisiveResult:
        """决战主状态机循环。"""
        _handlers = {
            DecisivePhase.ENTER_MAP: self._handle_enter_map,
            DecisivePhase.WAITING_FOR_MAP: self._handle_waiting_for_map,
            DecisivePhase.USE_LAST_FLEET: self._handle_use_last_fleet,
            DecisivePhase.DOCK_FULL: self._handle_dock_full,
            DecisivePhase.CHOOSE_FLEET: self._handle_choose_fleet,
            DecisivePhase.ADVANCE_CHOICE: self._handle_advance_choice,
            DecisivePhase.PREPARE_COMBAT: self._handle_prepare_combat,
            DecisivePhase.IN_COMBAT: self._handle_combat,
            DecisivePhase.NODE_RESULT: self._handle_node_result,
            DecisivePhase.STAGE_CLEAR: self._handle_stage_clear,
        }

        while self._state.phase != DecisivePhase.FINISHED:
            phase = self._state.phase

            # WAITING_FOR_MAP 太频繁，只在非等待阶段打日志
            if phase != DecisivePhase.WAITING_FOR_MAP:
                _log.debug(
                    '[决战] 阶段: {} | 小关: {} | 节点: {}',
                    phase.name,
                    self._state.stage,
                    self._state.node,
                )

            if phase == DecisivePhase.CHAPTER_CLEAR:
                _log.info('[决战] 大关通关!')
                self._state.phase = DecisivePhase.FINISHED
                return DecisiveResult.CHAPTER_CLEAR

            if phase == DecisivePhase.RETREAT:
                self._execute_retreat()
                self._state.reset()
                self._state.phase = DecisivePhase.ENTER_MAP
                continue

            if phase == DecisivePhase.LEAVE:
                self._execute_leave()
                self._state.phase = DecisivePhase.FINISHED
                return DecisiveResult.LEAVE

            handler = _handlers.get(phase)
            if handler is None:
                _log.error('[决战] 未知阶段: {}', phase)
                self._state.phase = DecisivePhase.FINISHED
                return DecisiveResult.ERROR

            handler()

        return DecisiveResult.CHAPTER_CLEAR



"""最小示例 — 修改舰队。

修改第 2 舰队的舰船配置。
"""

from autowsgr.ops import goto_page
from autowsgr.scheduler import launch
from autowsgr.ui import BattlePreparationPage, PageName


# 1. 启动 (加载配置 → 连接模拟器 → 启动游戏)
ctx = launch('usersettings.yaml')

goto_page(ctx, PageName.BATTLE_PREP)

page = BattlePreparationPage(ctx)

page.change_fleet(2, ['U-47', 'U-96'])



"""游戏「点击进入」启动画面 UI 控制器。

游戏冷启动后、进入主流程前会停在此画面，需要点击右下角按钮才能继续。

使用方式::

    from autowsgr.ui.start_screen_page import StartScreenPage

    screen = ctrl.screenshot()
    if StartScreenPage.is_current_page(screen):
        StartScreenPage(ctrl).click_enter()
        # 之后开始检测登录浮层
"""

from __future__ import annotations

import time
from typing import TYPE_CHECKING

from autowsgr.infra.logger import get_logger
from autowsgr.vision import MatchStrategy, PixelChecker, PixelRule, PixelSignature


if TYPE_CHECKING:
    import numpy as np

    from autowsgr.emulator import AndroidController


_log = get_logger('ui')

# ═══════════════════════════════════════════════════════════════════════════════
# 页面识别签名
# ═══════════════════════════════════════════════════════════════════════════════

PAGE_SIGNATURE = PixelSignature(
    name='启动画面',
    strategy=MatchStrategy.ALL,
    rules=[
        PixelRule.of(0.0508, 0.8722, (204, 210, 208), tolerance=30.0),
        PixelRule.of(0.8125, 0.8569, (224, 183, 41), tolerance=30.0),
        PixelRule.of(0.8695, 0.9028, (222, 198, 40), tolerance=30.0),
        PixelRule.of(0.9641, 0.8681, (222, 197, 43), tolerance=30.0),
        PixelRule.of(0.7914, 0.7333, (237, 237, 237), tolerance=30.0),
    ],
)
"""启动画面像素签名 — 底部横幅暖黄色调特征。"""


# ═══════════════════════════════════════════════════════════════════════════════
# 常量
# ═══════════════════════════════════════════════════════════════════════════════

#: 「点击进入」按钮坐标（右下角）
CLICK_ENTER: tuple[float, float] = (0.9, 0.85)

#: 点击后等待画面稳定的时间（秒）
_CLICK_SETTLE: float = 2.0


# ═══════════════════════════════════════════════════════════════════════════════
# 页面控制器
# ═══════════════════════════════════════════════════════════════════════════════


class StartScreenPage:
    """游戏「点击进入」启动画面控制器。

    **状态查询** 为 ``staticmethod``，只需截图即可调用。
    **操作动作** 为实例方法，通过注入的控制器执行。

    Parameters
    ----------
    ctrl:
        Android 设备控制器实例。
    """

    def __init__(self, ctrl: AndroidController) -> None:
        self._ctrl = ctrl

    # ── 页面识别 ──────────────────────────────────────────────────────────

    @staticmethod
    def is_current_page(screen: np.ndarray) -> bool:
        """判断截图是否为启动画面。

        通过底部横幅暖黄色调像素签名匹配判定。

        Parameters
        ----------
        screen:
            截图 (HxWx3, RGB)。
        """
        return PixelChecker.check_signature(screen, PAGE_SIGNATURE).matched

    # ── 操作动作 ──────────────────────────────────────────────────────────

    def click_enter(self) -> None:
        """点击右下角「点击进入」按钮，进入游戏主流程。

        点击坐标为 :data:`CLICK_ENTER` ``(0.9, 0.85)``，点击后等待
        :data:`_CLICK_SETTLE` 秒让画面稳定，之后可开始检测登录浮层。
        """
        _log.info('[UI] 点击「点击进入」按钮 {}', CLICK_ENTER)
        self._ctrl.click(*CLICK_ENTER)
        time.sleep(_CLICK_SETTLE)



"""UI 导航图 — 基于页面控制器函数的声明式路径查找。

每条边存储一个 **动作函数** ``action(ctx)``，内部调用对应页面控制器的
``navigate_to`` / ``go_back`` 等方法完成导航。
坐标、重试、截图验证均由页面控制器自行处理，本模块仅描述拓扑。

Usage::

    from autowsgr.ui.navigation import find_path

    path = find_path(PageName.MAIN, PageName.BUILD)
    for edge in path:
        edge.action(ctx)
"""

from __future__ import annotations

from collections import deque
from dataclasses import dataclass, field
from typing import TYPE_CHECKING

from autowsgr.types import PageName


if TYPE_CHECKING:
    from collections.abc import Callable

    from autowsgr.context import GameContext


# ═══════════════════════════════════════════════════════════════════════════════
# 导航边
# ═══════════════════════════════════════════════════════════════════════════════


@dataclass(frozen=True)
class NavEdge:
    """导航图中的一条有向边。

    Attributes
    ----------
    source:
        出发页面。
    target:
        到达页面。
    action:
        执行导航的函数 ``(ctx) -> None``，内部调用页面控制器方法。
    description:
        人类可读描述。
    """

    source: PageName
    target: PageName
    action: Callable[[GameContext], None] = field(repr=False)
    description: str = ''


# ═══════════════════════════════════════════════════════════════════════════════
# 动作函数 — 延迟导入避免循环依赖
# ═══════════════════════════════════════════════════════════════════════════════


def _main_to_map(ctx: GameContext) -> None:
    from autowsgr.ui.main_page import MainPage

    MainPage(ctx).navigate_to(MainPage.Target.SORTIE)


def _main_to_mission(ctx: GameContext) -> None:
    from autowsgr.ui.main_page import MainPage

    MainPage(ctx).navigate_to(MainPage.Target.TASK)


def _main_to_backyard(ctx: GameContext) -> None:
    from autowsgr.ui.main_page import MainPage

    MainPage(ctx).navigate_to(MainPage.Target.HOME)


def _main_to_sidebar(ctx: GameContext) -> None:
    from autowsgr.ui.main_page import MainPage

    MainPage(ctx).navigate_to(MainPage.Target.SIDEBAR)


def _map_to_main(ctx: GameContext) -> None:
    from autowsgr.ui.map.page import MapPage

    MapPage(ctx).go_back()


def _mission_to_main(ctx: GameContext) -> None:
    from autowsgr.ui.mission_page import MissionPage

    MissionPage(ctx).go_back()


def _backyard_to_main(ctx: GameContext) -> None:
    from autowsgr.ui.backyard_page import BackyardPage

    BackyardPage(ctx).go_back()


def _sidebar_to_main(ctx: GameContext) -> None:
    from autowsgr.ui.sidebar_page import SidebarPage

    SidebarPage(ctx).close()


def _map_to_decisive(ctx: GameContext) -> None:
    from autowsgr.ui.map.page import MapPage

    MapPage(ctx).enter_decisive()


def _battle_prep_to_map(ctx: GameContext) -> None:
    from autowsgr.ui.battle.preparation import BattlePreparationPage

    BattlePreparationPage(ctx).go_back()


def _backyard_to_bath(ctx: GameContext) -> None:
    from autowsgr.ui.backyard_page import BackyardPage, BackyardTarget

    BackyardPage(ctx).navigate_to(BackyardTarget.BATH)


def _backyard_to_canteen(ctx: GameContext) -> None:
    from autowsgr.ui.backyard_page import BackyardPage, BackyardTarget

    BackyardPage(ctx).navigate_to(BackyardTarget.CANTEEN)


def _bath_to_backyard(ctx: GameContext) -> None:
    from autowsgr.ui.bath_page import BathPage

    BathPage(ctx).go_back()


def _canteen_to_backyard(ctx: GameContext) -> None:
    from autowsgr.ui.canteen_page import CanteenPage

    CanteenPage(ctx).go_back()


def _sidebar_to_build(ctx: GameContext) -> None:
    from autowsgr.ui.sidebar_page import SidebarPage, SidebarTarget

    SidebarPage(ctx).navigate_to(SidebarTarget.BUILD)


def _sidebar_to_intensify(ctx: GameContext) -> None:
    from autowsgr.ui.sidebar_page import SidebarPage, SidebarTarget

    SidebarPage(ctx).navigate_to(SidebarTarget.INTENSIFY)


def _sidebar_to_friend(ctx: GameContext) -> None:
    from autowsgr.ui.sidebar_page import SidebarPage, SidebarTarget

    SidebarPage(ctx).navigate_to(SidebarTarget.FRIEND)


def _build_to_sidebar(ctx: GameContext) -> None:
    from autowsgr.ui.build_page import BuildPage

    BuildPage(ctx).go_back()


def _intensify_to_sidebar(ctx: GameContext) -> None:
    from autowsgr.ui.intensify_page import IntensifyPage

    IntensifyPage(ctx).go_back()


def _friend_to_sidebar(ctx: GameContext) -> None:
    from autowsgr.ui.friend_page import FriendPage

    FriendPage(ctx).go_back()


def _decisive_to_main(ctx: GameContext) -> None:
    from autowsgr.ui.decisive import DecisiveBattlePage

    DecisiveBattlePage(ctx).go_back()


def _main_to_event(ctx: GameContext) -> None:
    from autowsgr.ui.main_page import MainPage

    MainPage(ctx).navigate_to(MainPage.Target.EVENT)


def _event_to_main(ctx: GameContext) -> None:
    from autowsgr.ui.event.event_page import BaseEventPage

    BaseEventPage(ctx).go_back()


# ═══════════════════════════════════════════════════════════════════════════════
# 导航图
# ═══════════════════════════════════════════════════════════════════════════════

NAV_GRAPH: list[NavEdge] = [
    # ── 主页面 ↔ 一级页面 ──
    NavEdge(PageName.MAIN, PageName.MAP, _main_to_map, '主页面 → 地图'),
    NavEdge(PageName.MAIN, PageName.MISSION, _main_to_mission, '主页面 → 任务'),
    NavEdge(PageName.MAIN, PageName.BACKYARD, _main_to_backyard, '主页面 → 后院'),
    NavEdge(PageName.MAIN, PageName.SIDEBAR, _main_to_sidebar, '主页面 → 侧边栏'),
    NavEdge(PageName.MAP, PageName.MAIN, _map_to_main, '地图 → 主页面'),
    NavEdge(PageName.MISSION, PageName.MAIN, _mission_to_main, '任务 → 主页面'),
    NavEdge(PageName.BACKYARD, PageName.MAIN, _backyard_to_main, '后院 → 主页面'),
    NavEdge(PageName.SIDEBAR, PageName.MAIN, _sidebar_to_main, '侧边栏 → 主页面'),
    # ── 地图 → 子页面 ──
    NavEdge(PageName.MAP, PageName.DECISIVE_BATTLE, _map_to_decisive, '地图 → 决战'),
    # ── 出征准备 → 地图 ──
    NavEdge(PageName.BATTLE_PREP, PageName.MAP, _battle_prep_to_map, '出征准备 → 地图'),
    # ── 后院 ↔ 子页面 ──
    NavEdge(PageName.BACKYARD, PageName.BATH, _backyard_to_bath, '后院 → 浴室'),
    NavEdge(PageName.BACKYARD, PageName.CANTEEN, _backyard_to_canteen, '后院 → 食堂'),
    NavEdge(PageName.BATH, PageName.BACKYARD, _bath_to_backyard, '浴室 → 后院'),
    NavEdge(PageName.CANTEEN, PageName.BACKYARD, _canteen_to_backyard, '食堂 → 后院'),
    # ── 侧边栏 ↔ 子页面 ──
    NavEdge(PageName.SIDEBAR, PageName.BUILD, _sidebar_to_build, '侧边栏 → 建造'),
    NavEdge(PageName.SIDEBAR, PageName.INTENSIFY, _sidebar_to_intensify, '侧边栏 → 强化'),
    NavEdge(PageName.SIDEBAR, PageName.FRIEND, _sidebar_to_friend, '侧边栏 → 好友'),
    NavEdge(PageName.BUILD, PageName.SIDEBAR, _build_to_sidebar, '建造 → 侧边栏'),
    NavEdge(PageName.INTENSIFY, PageName.SIDEBAR, _intensify_to_sidebar, '强化 → 侧边栏'),
    NavEdge(PageName.FRIEND, PageName.SIDEBAR, _friend_to_sidebar, '好友 → 侧边栏'),
    # ── 决战 → 主页面 (跨级) ──
    NavEdge(PageName.DECISIVE_BATTLE, PageName.MAIN, _decisive_to_main, '决战 → 主页面'),
    # ── 活动 ↔ 主页面 ──
    NavEdge(PageName.MAIN, PageName.EVENT_MAP, _main_to_event, '主页面 → 活动'),
    NavEdge(PageName.EVENT_MAP, PageName.MAIN, _event_to_main, '活动 → 主页面'),
]


# ═══════════════════════════════════════════════════════════════════════════════
# 路径查找
# ═══════════════════════════════════════════════════════════════════════════════

_adjacency: dict[PageName, list[NavEdge]] = {}
for _e in NAV_GRAPH:
    _adjacency.setdefault(_e.source, []).append(_e)


def find_path(source: str, target: str) -> list[NavEdge] | None:
    """BFS 查找从 *source* 到 *target* 的最短路径。

    Parameters
    ----------
    source, target:
        页面名称，可以是 :class:`PageName` 或等价字符串。

    Returns
    -------
    list[NavEdge] | None
        路径上的边列表；``source == target`` 时返回空列表；不可达返回 ``None``。
    """
    if source == target:
        return []

    visited: set[str] = {source}
    queue: deque[tuple[str, list[NavEdge]]] = deque([(source, [])])

    while queue:
        current, path = queue.popleft()
        for edge in _adjacency.get(current, []):
            if edge.target in visited:
                continue
            new_path = [*path, edge]
            if edge.target == target:
                return new_path
            visited.add(edge.target)
            queue.append((edge.target, new_path))

    return None



"""UI 工具函数包。

汇总导航工具与选船列表 OCR 工具，对外保持与原 ``autowsgr.ui.utils`` 完全一致的公共 API。
"""

from .navigation import (
    DEFAULT_NAV_CONFIG,
    NavConfig,
    NavigationError,
    click_and_wait_for_page,
    click_and_wait_leave_page,
    confirm_operation,
    wait_for_page,
    wait_leave_page,
)
from .ship_list import (
    LEGACY_HEIGHT,
    LEGACY_LIST_WIDTH,
    LEGACY_WIDTH,
    locate_ship_rows,
    recognize_ships_in_list,
    to_legacy_format,
)


__all__ = [
    'DEFAULT_NAV_CONFIG',
    'LEGACY_HEIGHT',
    'LEGACY_LIST_WIDTH',
    'LEGACY_WIDTH',
    'NavConfig',
    'NavigationError',
    'click_and_wait_for_page',
    'click_and_wait_leave_page',
    'confirm_operation',
    'locate_ship_rows',
    'recognize_ships_in_list',
    'to_legacy_format',
    'wait_for_page',
    'wait_leave_page',
]



"""测试配置系统与日志工具。"""

from pathlib import Path

import pytest

from autowsgr.infra import (
    BattleConfig,
    ConfigManager,
    DecisiveConfig,
    EmulatorConfig,
    FightConfig,
    UserConfig,
)
from autowsgr.types import (
    DestroyShipWorkMode,
    EmulatorType,
    RepairMode,
)


# ── EmulatorConfig ──


class TestEmulatorConfig:
    def test_from_dict(self):
        cfg = EmulatorConfig.model_validate({'type': '蓝叠', 'serial': '127.0.0.1:5555'})
        assert cfg.type == EmulatorType.bluestacks
        assert cfg.serial == '127.0.0.1:5555'


# ── DecisiveBattleConfig ──


class TestDecisiveConfig:
    def test_invalid_chapter(self):
        with pytest.raises(ValueError):
            DecisiveConfig(chapter=0)


# ── UserConfig ──


class TestUserConfig:
    def test_from_yaml(self, tmp_yaml):
        content = """\
emulator:
  type: "蓝叠"
  serial: "127.0.0.1:5555"
  path: "C:/fake/player.exe"
account:
  game_app: "官服"
delay: 2.0
dock_full_destroy: false
"""
        path = tmp_yaml('config.yaml', content)
        cfg = UserConfig.from_yaml(path)
        assert cfg.emulator.type == EmulatorType.bluestacks
        assert cfg.emulator.serial == '127.0.0.1:5555'
        assert cfg.delay == 2.0
        assert cfg.dock_full_destroy is False

    def test_with_daily_automation(self, tmp_yaml):
        content = """\
emulator:
  type: "雷电"
  serial: "emulator-5554"
  path: "C:/fake/dnplayer.exe"
daily_automation:
  auto_exercise: false
  battle_type: "简单航母"
"""
        path = tmp_yaml('daily.yaml', content)
        cfg = UserConfig.from_yaml(path)
        assert cfg.daily_automation is not None
        assert cfg.daily_automation.auto_exercise is False
        assert cfg.daily_automation.battle_type == '简单航母'

    def test_with_decisive_battle(self, tmp_yaml):
        content = """\
emulator:
  type: "雷电"
  serial: "emulator-5554"
  path: "C:/fake/dnplayer.exe"
decisive_battle:
  chapter: 5
  repair_level: 2
"""
        path = tmp_yaml('decisive.yaml', content)
        cfg = UserConfig.from_yaml(path)
        assert cfg.decisive_battle is not None
        assert cfg.decisive_battle.chapter == 5
        assert cfg.decisive_battle.repair_level == 2

    def test_destroy_ship_config(self, tmp_yaml):
        content = """\
emulator:
  type: "雷电"
  serial: "emulator-5554"
  path: "C:/fake/dnplayer.exe"
destroy_ship_work_mode: 1
destroy_ship_types:
  - "驱逐"
  - "轻巡"
"""
        path = tmp_yaml('destroy.yaml', content)
        cfg = UserConfig.from_yaml(path)
        assert cfg.destroy_ship_work_mode == DestroyShipWorkMode.include
        assert len(cfg.destroy_ship_types) == 2


# ── FightConfig ──


class TestFightConfig:
    def test_repair_mode_expanded(self):
        cfg = FightConfig(repair_mode=RepairMode.moderate_damage)
        assert isinstance(cfg.repair_mode, list)
        assert len(cfg.repair_mode) == 6
        assert all(r == RepairMode.moderate_damage for r in cfg.repair_mode)

    def test_repair_mode_list_kept(self):
        modes = [RepairMode.moderate_damage, RepairMode.severe_damage] + [
            RepairMode.moderate_damage
        ] * 4
        cfg = FightConfig(repair_mode=modes)
        assert cfg.repair_mode == modes


class TestBattleConfig:
    def test_default_repair_mode(self):
        cfg = BattleConfig()
        assert isinstance(cfg.repair_mode, list)
        assert all(r == RepairMode.moderate_damage for r in cfg.repair_mode)


# ── ConfigManager ──


class TestConfigManager:
    def test_load_existing_file(self, tmp_yaml):
        content = """\
emulator:
  type: "MuMu"
  serial: "127.0.0.1:16384"
  path: "C:/fake/MuMuPlayer.exe"
delay: 2.5
"""
        path = tmp_yaml('settings.yaml', content)
        cfg = ConfigManager.load(path)
        assert cfg.emulator.type == EmulatorType.mumu
        assert cfg.delay == 2.5

    def test_load_nonexistent_returns_default(self, tmp_path: Path):
        cfg = ConfigManager.load(tmp_path / 'no_such_file.yaml')
        assert isinstance(cfg, UserConfig)
        assert cfg.delay == 1.5


# ── LogConfig (setup_logger) ──


class TestSetupLogger:
    """setup_logger 进行基本函数验证。"""

    def test_with_log_dir(self, tmp_path: Path):
        """log_dir 应被自动创建。"""
        from autowsgr.infra import setup_logger

        log_dir = tmp_path / 'logs' / 'sub'
        setup_logger(log_dir=log_dir, level='INFO')
        assert log_dir.exists()



"""异步任务管理 — 后台执行战斗任务并管理状态。"""

from __future__ import annotations

import asyncio
import threading
import uuid
from dataclasses import dataclass, field
from datetime import UTC, datetime
from enum import Enum
from typing import TYPE_CHECKING, Any

from autowsgr.infra.logger import get_logger
from autowsgr.server.ws_manager import ws_manager


if TYPE_CHECKING:
    from collections.abc import Callable


_log = get_logger('server.task')


class TaskStatus(Enum):
    """任务状态。"""

    IDLE = 'idle'
    RUNNING = 'running'
    COMPLETED = 'completed'
    FAILED = 'failed'
    STOPPED = 'stopped'


@dataclass
class TaskInfo:
    """任务信息。"""

    task_id: str
    task_type: str
    status: TaskStatus = TaskStatus.IDLE
    created_at: str = field(default_factory=lambda: datetime.now(UTC).isoformat())
    started_at: str | None = None
    finished_at: str | None = None

    # 进度
    current_round: int = 0
    total_rounds: int = 0
    current_node: str | None = None

    # 结果
    results: list[dict[str, Any]] = field(default_factory=list)
    error: str | None = None

    # 控制
    stop_requested: bool = False

    @property
    def progress(self) -> dict[str, Any]:
        """返回进度信息。"""
        return {
            'current': self.current_round,
            'total': self.total_rounds,
            'node': self.current_node,
        }

    @property
    def result_summary(self) -> dict[str, Any]:
        """返回结果摘要。"""
        return {
            'total_runs': self.total_rounds,
            'success_runs': len([r for r in self.results if r.get('success', False)]),
            'details': self.results,
        }


class TaskManager:
    """任务管理器。

    管理任务的创建、执行、状态查询和取消。
    所有战斗操作在后台线程执行，避免阻塞事件循环。
    """

    def __init__(self) -> None:
        self._current_task: TaskInfo | None = None
        self._executor_thread: threading.Thread | None = None
        self._stop_event = threading.Event()
        self._lock = threading.Lock()
        self._loop: asyncio.AbstractEventLoop | None = None

    @property
    def current_task(self) -> TaskInfo | None:
        """当前任务。"""
        return self._current_task

    @property
    def is_running(self) -> bool:
        """是否有任务正在运行。"""
        return self._current_task is not None and self._current_task.status == TaskStatus.RUNNING

    def set_loop(self, loop: asyncio.AbstractEventLoop) -> None:
        """设置事件循环引用，用于从线程中调用 async 函数。"""
        self._loop = loop

    def start_task(
        self,
        task_type: str,
        total_rounds: int,
        executor: Callable[[TaskInfo], list[dict[str, Any]]],
    ) -> str:
        """启动新任务。

        Parameters
        ----------
        task_type:
            任务类型标识
        total_rounds:
            总轮次
        executor:
            执行函数，接收 TaskInfo，返回结果列表

        Returns
        -------
        str
            任务 ID
        """
        with self._lock:
            if self.is_running:
                raise RuntimeError('已有任务正在运行')

            task_id = f'task_{uuid.uuid4().hex[:8]}'
            self._current_task = TaskInfo(
                task_id=task_id,
                task_type=task_type,
                status=TaskStatus.RUNNING,
                total_rounds=total_rounds,
                started_at=datetime.now(UTC).isoformat(),
            )
            self._stop_event.clear()

            # 启动后台线程执行
            self._executor_thread = threading.Thread(
                target=self._run_in_thread,
                args=(executor,),
                daemon=True,
            )
            self._executor_thread.start()

            _log.info('[Task] 启动任务: {} ({})', task_id, task_type)
            return task_id

    def _run_in_thread(
        self,
        executor: Callable[[TaskInfo], list[dict[str, Any]]],
    ) -> None:
        """在后台线程中执行任务。"""
        assert self._current_task is not None
        task = self._current_task

        try:
            results = executor(task)

            # 检查是否被请求停止
            if task.stop_requested:
                task.status = TaskStatus.STOPPED
            else:
                task.status = TaskStatus.COMPLETED
                task.results = results

            task.finished_at = datetime.now(UTC).isoformat()
            _log.info('[Task] 任务完成: {} ({})', task.task_id, task.status.value)

        except Exception as e:
            task.status = TaskStatus.FAILED
            task.error = str(e)
            task.finished_at = datetime.now(UTC).isoformat()
            _log.error('[Task] 任务失败: {} - {}', task.task_id, e)

        finally:
            # 通过事件循环发送 WebSocket 通知
            if self._loop:
                asyncio.run_coroutine_threadsafe(
                    self._notify_completion(task),
                    self._loop,
                )

    async def _notify_completion(self, task: TaskInfo) -> None:
        """发送任务完成通知。"""
        success = task.status == TaskStatus.COMPLETED
        result = task.result_summary if success else None
        await ws_manager.send_task_completed(
            task_id=task.task_id,
            success=success,
            result=result,
            error=task.error,
        )

    def stop_task(self) -> bool:
        """请求停止当前任务。"""
        with self._lock:
            if not self.is_running or self._current_task is None:
                return False

            self._current_task.stop_requested = True
            self._stop_event.set()
            _log.info('[Task] 请求停止任务: {}', self._current_task.task_id)
            return True

    def update_progress(
        self,
        current_round: int | None = None,
        current_node: str | None = None,
    ) -> None:
        """更新任务进度 (从执行线程调用)。"""
        if self._current_task is None:
            return

        if current_round is not None:
            self._current_task.current_round = current_round
        if current_node is not None:
            self._current_task.current_node = current_node

        # 通过事件循环发送 WebSocket 更新
        if self._loop:
            asyncio.run_coroutine_threadsafe(
                ws_manager.send_task_update(
                    task_id=self._current_task.task_id,
                    status='running',
                    progress=self._current_task.progress,
                ),
                self._loop,
            )

    def add_result(self, result: dict[str, Any]) -> None:
        """添加一轮结果 (从执行线程调用)。"""
        if self._current_task is None:
            return
        self._current_task.results.append(result)

    def should_stop(self) -> bool:
        """检查是否应该停止 (从执行线程调用)。"""
        if self._current_task is None:
            return True
        return self._current_task.stop_requested

    def get_status(self) -> dict[str, Any]:
        """获取当前任务状态。"""
        if self._current_task is None:
            return {
                'task_id': None,
                'status': TaskStatus.IDLE.value,
                'progress': None,
                'result': None,
            }

        task = self._current_task
        result = None
        if task.status in (TaskStatus.COMPLETED, TaskStatus.FAILED, TaskStatus.STOPPED):
            result = task.result_summary if task.status == TaskStatus.COMPLETED else None

        return {
            'task_id': task.task_id,
            'status': task.status.value,
            'progress': task.progress if task.status == TaskStatus.RUNNING else None,
            'result': result,
            'error': task.error,
        }


# 全局单例
task_manager = TaskManager()



"""模拟器进程管理子包。

将各操作系统的模拟器进程管理器统一组织在此包中：

- :mod:`.base`    — 抽象基类 :class:`EmulatorProcessManager`
- :mod:`.windows` — :class:`WindowsEmulatorManager`
- :mod:`.macos`   — :class:`MacEmulatorManager`
- :mod:`.linux`   — :class:`LinuxEmulatorManager`

使用方式::

    from autowsgr.emulator.os_control import create_emulator_manager

    manager = create_emulator_manager(config)
    manager.start()
    manager.wait_until_online(timeout=120)
    manager.stop()
"""

from __future__ import annotations

from autowsgr.infra import EmulatorConfig, EmulatorError
from autowsgr.types import OSType

from .base import EmulatorProcessManager
from .linux import LinuxEmulatorManager
from .macos import MacEmulatorManager
from .windows import WindowsEmulatorManager


def create_emulator_manager(
    config: EmulatorConfig,
    os_type: OSType | None = None,
) -> EmulatorProcessManager:
    """根据当前操作系统创建对应的模拟器进程管理器。

    Parameters
    ----------
    config:
        模拟器配置。
    os_type:
        操作系统类型，为 None 时自动检测。

    Returns
    -------
    EmulatorProcessManager
        对应操作系统的管理器实例。

    Raises
    ------
    EmulatorError
        不支持的操作系统。
    """
    if os_type is None:
        os_type = OSType.auto()

    match os_type:
        case OSType.windows:
            return WindowsEmulatorManager(config)
        case OSType.macos:
            return MacEmulatorManager(config)
        case OSType.linux:
            return LinuxEmulatorManager(config)
        case _:
            raise EmulatorError(f'不支持的操作系统: {os_type}')


__all__ = [
    'EmulatorProcessManager',
    'LinuxEmulatorManager',
    'MacEmulatorManager',
    'WindowsEmulatorManager',
    'create_emulator_manager',
]



"""侧边栏页面 UI 控制器端到端测试。

运行方式::

    python testing/ui/sidebar_page/e2e.py [serial] [--auto] [--debug]

前置条件：
    游戏位于 **侧边栏** (主页面 → ≡ 菜单)

测试内容：
    1. 验证初始状态 (侧边栏识别)
    2. 侧边栏 → 建造页面 (4 标签切换) → ◁ 侧边栏
    3. 侧边栏 → 强化页面 (3 标签切换) → ◁ 侧边栏
    4. 侧边栏 → 好友页面 → ◁ 侧边栏
    5. 侧边栏 → close → 主页面
"""

from __future__ import annotations

import sys
from pathlib import Path


sys.path.insert(0, str(Path(__file__).parent.parent.parent.parent))

from testing.ui._framework import (
    UIControllerTestRunner,
    connect_via_launcher,
    ensure_page,
    info,
    parse_e2e_args,
    reset_to_main_page,
)


def run_test(runner: UIControllerTestRunner) -> None:
    from autowsgr.ui.build_page import BuildPage, BuildTab
    from autowsgr.ui.friend_page import FriendPage
    from autowsgr.ui.intensify_page import IntensifyPage, IntensifyTab
    from autowsgr.ui.main_page import MainPage
    from autowsgr.ui.sidebar_page import SidebarPage, SidebarTarget

    sidebar_page = SidebarPage(runner.ctx)
    build_page = BuildPage(runner.ctx)
    intensify_page = IntensifyPage(runner.ctx)
    friend_page = FriendPage(runner.ctx)

    # Step 0: 验证初始
    runner.verify_current('初始验证: 侧边栏', '侧边栏', SidebarPage.is_current_page)
    if runner.aborted:
        return

    # Step 1: 侧边栏 → 建造
    runner.execute_step(
        '侧边栏 → 建造页面',
        '建造页面',
        BuildPage.is_current_page,
        lambda: sidebar_page.navigate_to(SidebarTarget.BUILD),
    )
    if runner.aborted:
        return

    # Step 2-4: 建造页面内标签切换
    for tab in [BuildTab.DESTROY, BuildTab.DEVELOP, BuildTab.DISCARD, BuildTab.BUILD]:
        runner.execute_step(
            f'建造: 切换标签 → {tab.value}',
            '建造页面',
            BuildPage.is_current_page,
            lambda t=tab: build_page.switch_tab(t),
        )
        if runner.aborted:
            return

    # Step 5: 建造 → ◁ 侧边栏
    runner.execute_step(
        '建造页面 → ◁ 侧边栏',
        '侧边栏',
        SidebarPage.is_current_page,
        lambda: build_page.go_back(),
    )
    if runner.aborted:
        return

    # Step 6: 侧边栏 → 强化
    runner.execute_step(
        '侧边栏 → 强化页面',
        '强化页面',
        IntensifyPage.is_current_page,
        lambda: sidebar_page.navigate_to(SidebarTarget.INTENSIFY),
    )
    if runner.aborted:
        return

    # Step 7-8: 强化页面内标签切换
    for tab in [IntensifyTab.REMAKE, IntensifyTab.SKILL, IntensifyTab.INTENSIFY]:
        runner.execute_step(
            f'强化: 切换标签 → {tab.value}',
            '强化页面',
            IntensifyPage.is_current_page,
            lambda t=tab: intensify_page.switch_tab(t),
        )
        if runner.aborted:
            return

    # Step 9: 强化 → ◁ 侧边栏
    runner.execute_step(
        '强化页面 → ◁ 侧边栏',
        '侧边栏',
        SidebarPage.is_current_page,
        lambda: intensify_page.go_back(),
    )
    if runner.aborted:
        return

    # Step 10: 侧边栏 → 好友
    runner.execute_step(
        '侧边栏 → 好友页面',
        '好友页面',
        FriendPage.is_current_page,
        lambda: sidebar_page.navigate_to(SidebarTarget.FRIEND),
    )
    if runner.aborted:
        return

    # Step 11: 好友 → ◁ 侧边栏
    runner.execute_step(
        '好友页面 → ◁ 侧边栏',
        '侧边栏',
        SidebarPage.is_current_page,
        lambda: friend_page.go_back(),
    )
    if runner.aborted:
        return

    # Step 12: 侧边栏 → close → 主页面
    runner.execute_step(
        '侧边栏 → close → 主页面',
        '主页面',
        MainPage.is_current_page,
        lambda: sidebar_page.close(),
    )


def _navigate_to(ctrl, pause: float) -> None:
    """从任意已知页面导航到侧边栏。"""
    import time

    from autowsgr.context import GameContext
    from autowsgr.infra import UserConfig
    from autowsgr.ui.main_page import MainPage

    if not reset_to_main_page(ctrl, pause):
        return
    screen = ctrl.screenshot()
    if MainPage.is_current_page(screen):
        ctx = GameContext(ctrl=ctrl, config=UserConfig())
        MainPage(ctx).navigate_to(MainPage.Target.SIDEBAR)
        time.sleep(pause)


def main() -> None:
    args = parse_e2e_args(
        '侧边栏 (SidebarPage) e2e 测试',
        precondition='游戏位于侧边栏 (主页面 → ≡)',
        default_log_dir='logs/e2e/sidebar_page',
    )
    ctrl = connect_via_launcher(args.serial, args.log_dir, args.log_level)
    from loguru import logger

    logger.info('=== 侧边栏 e2e 测试开始 ===')
    from autowsgr.ui.sidebar_page import SidebarPage

    if not ensure_page(
        ctrl,
        SidebarPage.is_current_page,
        lambda: _navigate_to(ctrl, args.pause),
        '侧边栏',
        auto_mode=args.auto,
        pause=args.pause,
    ):
        ctrl.disconnect()
        sys.exit(1)
    runner = UIControllerTestRunner(
        ctrl,
        controller_name='侧边栏',
        log_dir=args.log_dir,
        auto_mode=args.auto,
        pause=args.pause,
    )
    try:
        run_test(runner)
    except KeyboardInterrupt:
        from testing.ui._framework import warn

        warn('用户中断')
    except Exception as exc:
        from testing.ui._framework import fail

        fail(f'未预期异常: {exc}')
        logger.opt(exception=True).error('侧边栏 e2e 测试异常')
    finally:
        runner.finalize()
        runner.print_summary()
        ctrl.disconnect()
        info('设备已断开')

    r = runner.report
    sys.exit(1 if (r.failed > 0 or r.errors > 0) else 0)


if __name__ == '__main__':
    main()



"""侧边栏页面 UI 控制器。

覆盖游戏 **侧边栏** (左下角 ≡ 菜单) 的导航交互。

已完成

使用方式::

    from autowsgr.ui.sidebar_page import SidebarPage, SidebarTarget

    page = SidebarPage(ctrl)

    # 页面识别
    screen = ctrl.screenshot()
    if SidebarPage.is_current_page(screen):
        page.navigate_to(SidebarTarget.BUILD)

    # 关闭侧边栏
    page.close()
"""

from __future__ import annotations

import enum
import time
from typing import TYPE_CHECKING

from autowsgr.infra.logger import get_logger
from autowsgr.types import PageName
from autowsgr.ui.utils import click_and_wait_for_page, wait_for_page
from autowsgr.vision import (
    Color,
    PixelChecker,
)


if TYPE_CHECKING:
    import numpy as np

    from autowsgr.context import GameContext


_log = get_logger('ui')

# ═══════════════════════════════════════════════════════════════════════════════
# 枚举
# ═══════════════════════════════════════════════════════════════════════════════


class SidebarTarget(enum.Enum):
    """侧边栏可导航的目标。"""

    BUILD = '建造'
    INTENSIFY = '强化'
    FRIEND = '好友'


# ═══════════════════════════════════════════════════════════════════════════════
# 页面识别签名
# ═══════════════════════════════════════════════════════════════════════════════

MENU_PROBES: list[tuple[float, float]] = [
    (0.0417, 0.0806),  # 商城
    (0.0422, 0.2102),  # 活动
    (0.0453, 0.3463),  # 建造
    (0.0406, 0.4676),  # 强化
    (0.0396, 0.6028),  # 图鉴
    (0.0432, 0.7231),  # 好友
]
"""侧边栏左侧 6 个菜单探测点 (来自 sig.py)。

每个点在未选中时为深灰 ≈ (57, 57, 57)，选中时为亮蓝 ≈ (0, 160, 232)。
"""

_MENU_GRAY = Color.of(57, 57, 57)
"""菜单项未选中颜色 (深灰)。"""
_MENU_SELECTED = Color.of(0, 160, 232)
"""菜单项选中颜色 (亮蓝)。"""
_MENU_TOLERANCE = 30.0
"""菜单项颜色匹配容差。"""


# ═══════════════════════════════════════════════════════════════════════════════
# 导航按钮点击坐标
# ═══════════════════════════════════════════════════════════════════════════════

CLICK_NAV: dict[SidebarTarget, tuple[float, float]] = {
    SidebarTarget.BUILD: (0.1563, 0.3704),
    SidebarTarget.INTENSIFY: (0.1563, 0.5000),
    SidebarTarget.FRIEND: (0.1563, 0.7593),
}
"""侧边栏菜单项点击坐标。

坐标换算: 旧代码 (150, 200) / (150, 270) / (150, 410) ÷ (960, 540)。
"""

CLICK_CLOSE: tuple[float, float] = (0.0438, 0.8963)
"""关闭侧边栏 (左下角 ≡ 同一切换按钮)。"""

CLICK_SUBMENU: dict[SidebarTarget, tuple[float, float]] = {
    SidebarTarget.BUILD: (0.375, 0.3704),
    SidebarTarget.INTENSIFY: (0.375, 0.5000),
}
"""二级弹出菜单点击坐标。

建造 和 强化 点击后会弹出子选项菜单 (如 建造/特别船坞)，
需要二次点击选中第一个选项。坐标来自旧代码 (360, 200)/(360, 270) ÷ (960, 540)。
"""

_SUBMENU_TARGETS: frozenset[SidebarTarget] = frozenset(
    {
        SidebarTarget.BUILD,
        SidebarTarget.INTENSIFY,
    }
)
"""需要二级菜单点击的导航目标。"""

SUBMENU_DELAY: float = 1.25
"""点击菜单项后等待二级菜单弹出的延迟 (秒)。"""


# ═══════════════════════════════════════════════════════════════════════════════
# 页面控制器
# ═══════════════════════════════════════════════════════════════════════════════


class SidebarPage:
    """侧边栏页面控制器。

    **状态查询** 为 ``staticmethod``，只需截图即可调用。
    **操作动作** 为实例方法，通过注入的控制器执行。

    Parameters
    ----------
    ctrl:
        Android 设备控制器实例。
    """

    def __init__(self, ctx: GameContext) -> None:
        self._ctx = ctx
        self._ctrl = ctx.ctrl

    # ── 页面识别 ──────────────────────────────────────────────────────────

    @staticmethod
    def is_current_page(screen: np.ndarray) -> bool:
        """判断截图是否为侧边栏页面。

        检测逻辑:
        1. 6 个左侧菜单探测点每个都必须匹配 **灰色** 或 **蓝色高亮**。
        2. 蓝色高亮的数量为 0 或 1 (无选中 / 单选中)。

        Parameters
        ----------
        screen:
            截图 (HxWx3, RGB)。
        """
        blue_count = 0
        for x, y in MENU_PROBES:
            pixel = PixelChecker.get_pixel(screen, x, y)
            if pixel.near(_MENU_SELECTED, _MENU_TOLERANCE):
                blue_count += 1
            elif pixel.near(_MENU_GRAY, _MENU_TOLERANCE):
                pass  # 灰色 — 正常
            else:
                return False  # 既不灰也不蓝 → 不是侧边栏
        return blue_count <= 1

    # ── 导航 ──────────────────────────────────────────────────────────────

    def navigate_to(self, target: SidebarTarget) -> None:
        """点击菜单项，进入指定子页面。

        建造 / 强化 需要二级菜单选择 (点击侧边栏项 → 等待弹出 → 点击子选项)。
        好友 直接单次点击。

        Parameters
        ----------
        target:
            导航目标。

        Raises
        ------
        NavigationError
            超时未到达目标页面。
        """
        from autowsgr.ui.build_page import BuildPage
        from autowsgr.ui.friend_page import FriendPage
        from autowsgr.ui.intensify_page import IntensifyPage

        target_checker = {
            SidebarTarget.BUILD: BuildPage.is_current_page,
            SidebarTarget.INTENSIFY: IntensifyPage.is_current_page,
            SidebarTarget.FRIEND: FriendPage.is_current_page,
        }
        _log.info('[UI] 侧边栏 → {}', target.value)

        if target in _SUBMENU_TARGETS:
            # 二级菜单: 点击侧边栏项 → 等弹出 → 点击子选项 → 验证
            self._navigate_with_submenu(
                target,
                target_checker[target],
            )
        else:
            # 单次点击 (好友)
            click_and_wait_for_page(
                self._ctrl,
                click_coord=CLICK_NAV[target],
                checker=target_checker[target],
                source=PageName.SIDEBAR,
                target=target.value,
            )

    def _navigate_with_submenu(
        self,
        target: SidebarTarget,
        checker,
    ) -> None:
        """带二级弹出菜单的导航 (建造 / 强化)。

        流程: 点击侧边栏项 → 等待弹出 → 点击子选项 → 验证到达目标页面。
        整个流程带重试。
        """
        from autowsgr.ui.utils import DEFAULT_NAV_CONFIG, NavigationError

        config = DEFAULT_NAV_CONFIG
        last_err: NavigationError | None = None

        for attempt in range(1, config.max_retries + 1):
            if attempt > 1:
                _log.warning(
                    '[UI] 二级菜单重试 {}/{}: 侧边栏 → {} (等 {:.1f}s)',
                    attempt,
                    config.max_retries,
                    target.value,
                    config.retry_delay,
                )
                time.sleep(config.retry_delay)

            # Step 1: 点击侧边栏菜单项
            self._ctrl.click(*CLICK_NAV[target])
            # Step 2: 等待二级弹出菜单出现
            time.sleep(SUBMENU_DELAY)
            # Step 3: 点击子选项
            self._ctrl.click(*CLICK_SUBMENU[target])

            try:
                wait_for_page(
                    self._ctrl,
                    checker,
                    timeout=config.timeout,
                    interval=config.interval,
                    handle_overlays=config.handle_overlays,
                    source=PageName.SIDEBAR,
                    target=target.value,
                )
                return
            except NavigationError as e:
                last_err = e
                _log.warning(
                    '[UI] 二级菜单后超时 ({}/{}): 侧边栏 → {}',
                    attempt,
                    config.max_retries,
                    target.value,
                )

        raise NavigationError(
            f'导航失败 (已重试 {config.max_retries} 次): 侧边栏 → {target.value}',
            screen=self._ctrl.screenshot(),
        ) from last_err

    def go_to_build(self) -> None:
        """点击「建造」— 进入建造页面。"""
        self.navigate_to(SidebarTarget.BUILD)

    def go_to_intensify(self) -> None:
        """点击「强化」— 进入强化页面。"""
        self.navigate_to(SidebarTarget.INTENSIFY)

    def go_to_friend(self) -> None:
        """点击「好友」— 进入好友页面。"""
        self.navigate_to(SidebarTarget.FRIEND)

    # ── 关闭 ──────────────────────────────────────────────────────────────

    def close(self) -> None:
        """关闭侧边栏，返回主页面。

        点击后反复截图验证，确认已到达主页面。

        Raises
        ------
        NavigationError
            超时未关闭侧边栏。
        """
        from autowsgr.ui.main_page import MainPage

        _log.info('[UI] 侧边栏 → 关闭 (返回主页面)')
        click_and_wait_for_page(
            self._ctrl,
            click_coord=CLICK_CLOSE,
            checker=MainPage.is_current_page,
            source=PageName.SIDEBAR,
            target=PageName.MAIN,
        )



"""健康检查路由 — /api/health"""

from __future__ import annotations

import time

from fastapi import APIRouter

from autowsgr.server.schemas import ApiResponse
from autowsgr.server.task_manager import task_manager

from .. import main as _main


router = APIRouter(tags=['health'])

_server_start_time = time.monotonic()


@router.get('/api/health', response_model=ApiResponse)
async def health_check():
    """健康检查端点。"""
    uptime = int(time.monotonic() - _server_start_time)
    task_info = None
    if task_manager.current_task and task_manager.is_running:
        task_info = {
            'task_id': task_manager.current_task.task_id,
            'status': task_manager.current_task.status.value,
        }

    return ApiResponse(
        success=True,
        data={
            'status': 'ok',
            'uptime_seconds': uptime,
            'emulator_connected': _main._ctx is not None,
            'current_task': task_info,
        },
    )



"""任务页面识别 - 按钮扫描、OCR 识别、模糊匹配。"""

from __future__ import annotations

from typing import TYPE_CHECKING

from autowsgr.ui.mission_page.data import (
    BUTTON_SCAN_ROI,
    CLAIM_B_MAX,
    CLAIM_G_MIN,
    CLAIM_R_MIN,
    CLUSTER_GAP,
    GOTO_COLOR,
    GOTO_TOLERANCE,
    MIN_CLUSTER_SIZE,
    NAME_CROP_MIN_Y,
    NAME_ROI_X1,
    NAME_ROI_X2,
    NAME_ROI_Y_PAD,
    NAME_Y_OFFSET,
    OCR_CONFIDENCE_MIN,
    PROGRESS_RE,
    PROGRESS_ROI_X1,
    PROGRESS_ROI_X2,
    PROGRESS_ROI_Y_PAD,
    PROGRESS_Y_OFFSET,
    SCAN_X,
    SCAN_Y_STEP,
    WIDE_BTN_CHECK_X,
    ButtonType,
    MissionInfo,
)
from autowsgr.vision import PixelChecker
from autowsgr.vision.roi import ROI


if TYPE_CHECKING:
    import numpy as np

    from autowsgr.vision import OCREngine
    from autowsgr.vision.pixel import Color


# ═══════════════════════════════════════════════════════════════════════════════
# 像素扫描
# ═══════════════════════════════════════════════════════════════════════════════


def _is_goto_color(c: Color) -> bool:
    return c.near(GOTO_COLOR, GOTO_TOLERANCE)


def _is_claim_color(c: Color) -> bool:
    return c.r >= CLAIM_R_MIN and c.g >= CLAIM_G_MIN and c.b <= CLAIM_B_MAX


def find_button_rows(screen: np.ndarray) -> list[tuple[float, ButtonType]]:
    """扫描截图右侧, 定位所有 "前往"/"领取" 按钮行。

    Returns
    -------
    list[tuple[float, ButtonType]]
        每个元素为 (y_center_relative, button_type), 按 y 从上到下排列。
    """
    y_start = BUTTON_SCAN_ROI.y1
    y_end = BUTTON_SCAN_ROI.y2

    # 收集所有匹配像素的 (y, type)
    hits: list[tuple[float, ButtonType]] = []
    y = y_start
    while y <= y_end:
        c = PixelChecker.get_pixel(screen, SCAN_X, y)
        if _is_goto_color(c):
            hits.append((y, ButtonType.GOTO))
        elif _is_claim_color(c):
            hits.append((y, ButtonType.CLAIM))
        y += SCAN_Y_STEP

    if not hits:
        return []

    # 聚类: 连续同类型像素归为一组
    clusters: list[list[tuple[float, ButtonType]]] = [[hits[0]]]
    for h in hits[1:]:
        prev = clusters[-1][-1]
        if h[0] - prev[0] < CLUSTER_GAP and h[1] == prev[1]:
            clusters[-1].append(h)
        else:
            clusters.append([h])

    # 取每个簇的中心 y 和类型, 过滤过小的簇 (噪点)
    result: list[tuple[float, ButtonType]] = []
    for cluster in clusters:
        if len(cluster) < MIN_CLUSTER_SIZE:
            continue
        ys = [p[0] for p in cluster]
        y_center = (min(ys) + max(ys)) / 2
        btn_type = cluster[0][1]
        result.append((y_center, btn_type))

    # 过滤底部 "一键领取" 按钮: 它的金色区域横向延伸到 x=0.80,
    # 而任务行按钮仅在 x=0.86-0.96 范围内
    filtered: list[tuple[float, ButtonType]] = []
    for y_center, btn_type in result:
        c_wide = PixelChecker.get_pixel(screen, WIDE_BTN_CHECK_X, y_center)
        if _is_goto_color(c_wide) or _is_claim_color(c_wide):
            continue  # 宽按钮 (UI 元素), 跳过
        # 过滤部分可见行: 名称裁切区域被页面标题栏遮挡
        name_top = y_center + NAME_Y_OFFSET - NAME_ROI_Y_PAD
        if name_top < NAME_CROP_MIN_Y:
            continue
        filtered.append((y_center, btn_type))

    return filtered


# ═══════════════════════════════════════════════════════════════════════════════
# OCR + 匹配
# ═══════════════════════════════════════════════════════════════════════════════


def recognize_row(
    screen: np.ndarray,
    anchor_y: float,
    btn_type: ButtonType,
    ocr: OCREngine,
    candidates: list[str],
) -> MissionInfo | None:
    """识别单行任务。返回 None 表示无效行 (部分可见/垃圾 OCR)。"""
    name_y = anchor_y + NAME_Y_OFFSET
    prog_y = anchor_y + PROGRESS_Y_OFFSET

    # -- 裁切名称区域并 OCR --
    name_roi = ROI(
        NAME_ROI_X1,
        max(0.0, name_y - NAME_ROI_Y_PAD),
        NAME_ROI_X2,
        min(1.0, name_y + NAME_ROI_Y_PAD),
    )
    name_img = name_roi.crop(screen)
    name_result = ocr.recognize_single(name_img)
    raw_text = name_result.text.strip()
    confidence = name_result.confidence

    # -- OCR 置信度过滤: 拒绝垃圾识别 (部分可见任务行) --
    if confidence < OCR_CONFIDENCE_MIN:
        return None

    # -- 模糊匹配数据库 --
    matched_name = match_mission_name(raw_text, candidates)
    final_name = matched_name or raw_text

    # -- 裁切进度区域并 OCR --
    prog_roi = ROI(
        PROGRESS_ROI_X1,
        max(0.0, prog_y - PROGRESS_ROI_Y_PAD),
        PROGRESS_ROI_X2,
        min(1.0, prog_y + PROGRESS_ROI_Y_PAD),
    )
    prog_img = prog_roi.crop(screen)
    prog_result = ocr.recognize_single(prog_img, allowlist='0123456789%:')
    prog_text = prog_result.text.strip()
    m = PROGRESS_RE.search(prog_text)
    progress = int(m.group(1)) if m else -1

    claimable = btn_type == ButtonType.CLAIM

    return MissionInfo(
        name=final_name,
        raw_text=raw_text,
        progress=progress,
        claimable=claimable,
        confidence=confidence,
    )


def match_mission_name(
    ocr_text: str,
    candidates: list[str],
    threshold: int = 5,
) -> str | None:
    """在任务数据库中模糊匹配任务名。

    策略:
    1. 精确匹配
    2. 子串包含
    3. Levenshtein 距离 <= threshold
    """
    if not ocr_text:
        return None

    # 精确匹配
    for name in candidates:
        if ocr_text == name:
            return name

    # 子串包含: 仅当长度比 >= 70% 时视为有效匹配, 优先最长候选
    substring_hits = [
        name
        for name in candidates
        if (name in ocr_text or ocr_text in name)
        and min(len(name), len(ocr_text)) >= 0.7 * max(len(name), len(ocr_text))
    ]
    if substring_hits:
        return max(substring_hits, key=len)

    # Levenshtein 模糊匹配
    from autowsgr.vision.ocr import _edit_distance

    best_name: str | None = None
    best_dist = threshold + 1
    for name in candidates:
        dist = _edit_distance(ocr_text, name)
        if dist < best_dist:
            best_dist = dist
            best_name = name
    return best_name if best_dist <= threshold else None



"""解装舰船操作。

导航到建造页面（解体标签）并委托 UI 层执行。

``ship_types=None`` 表示不过滤舰种，全部解装；
传入舰种列表则只解装指定舰种。
"""

from __future__ import annotations

from typing import TYPE_CHECKING

from autowsgr.infra.logger import get_logger
from autowsgr.ops.navigate import goto_page
from autowsgr.types import PageName, ShipType


if TYPE_CHECKING:
    from autowsgr.context import GameContext

_log = get_logger('ops')


def destroy_ships(
    ctx: GameContext,
    *,
    ship_types: list[ShipType] | None = None,
    remove_equipment: bool = True,
) -> None:
    """解装舰船。

    Parameters
    ----------
    ctx:
        游戏上下文。
    ship_types:
        要解装的舰种列表。``None`` (默认) 表示不过滤，直接快速全选解装全部。
    remove_equipment:
        是否在解装前卸下装备。默认 ``True``。
    """
    from autowsgr.ui.build_page import BuildPage, BuildTab

    _log.info('[OPS] 开始解装')
    goto_page(ctx, PageName.BUILD)

    page = BuildPage(ctx)
    page.switch_tab(BuildTab.DESTROY)
    page.destroy_ships(ship_types, remove_equipment=remove_equipment)

    goto_page(ctx, PageName.MAIN)
    _log.info('[OPS] 解装完成')



"""测试文件工具函数。"""

from pathlib import Path

import pytest

from autowsgr.infra import load_yaml, merge_dicts, save_yaml


class TestLoadYaml:
    """测试 load_yaml。"""

    def test_load_simple(self, tmp_yaml):
        p = tmp_yaml('test.yaml', 'key: value\ncount: 42\n')
        result = load_yaml(p)
        assert result == {'key': 'value', 'count': 42}

    def test_load_nested(self, tmp_yaml):
        content = 'a:\n  b:\n    c: 1\n'
        p = tmp_yaml('nested.yaml', content)
        result = load_yaml(p)
        assert result == {'a': {'b': {'c': 1}}}

    def test_load_empty_file(self, tmp_yaml):
        p = tmp_yaml('empty.yaml', '')
        result = load_yaml(p)
        assert result == {}

    def test_load_list(self, tmp_yaml):
        content = 'items:\n  - a\n  - b\n  - c\n'
        p = tmp_yaml('list.yaml', content)
        result = load_yaml(p)
        assert result == {'items': ['a', 'b', 'c']}

    def test_file_not_found(self):
        with pytest.raises(FileNotFoundError):
            load_yaml(Path('nonexistent_file.yaml'))

    def test_chinese_content(self, tmp_yaml):
        content = 'name: 胡德\ntype: 战巡\n'
        p = tmp_yaml('cn.yaml', content)
        result = load_yaml(p)
        assert result == {'name': '胡德', 'type': '战巡'}


class TestSaveYaml:
    """测试 save_yaml。"""

    def test_save_and_reload(self, tmp_path: Path):
        data = {'key': 'value', 'nested': {'a': 1}}
        path = tmp_path / 'output.yaml'
        save_yaml(data, path)
        assert path.exists()
        reloaded = load_yaml(path)
        assert reloaded == data

    def test_creates_parent_dirs(self, tmp_path: Path):
        path = tmp_path / 'deep' / 'nested' / 'dir' / 'config.yaml'
        save_yaml({'k': 'v'}, path)
        assert path.exists()

    def test_chinese_roundtrip(self, tmp_path: Path):
        data = {'舰船': ['胡德', '俾斯麦'], '类型': '战巡'}
        path = tmp_path / 'cn_out.yaml'
        save_yaml(data, path)
        reloaded = load_yaml(path)
        assert reloaded == data


class TestMergeDicts:
    """测试 merge_dicts。"""

    def test_simple_merge(self):
        base = {'a': 1, 'b': 2}
        override = {'b': 3, 'c': 4}
        result = merge_dicts(base, override)
        assert result == {'a': 1, 'b': 3, 'c': 4}

    def test_deep_merge(self):
        base = {'a': {'x': 1, 'y': 2}, 'b': 10}
        override = {'a': {'y': 99, 'z': 3}}
        result = merge_dicts(base, override)
        assert result == {'a': {'x': 1, 'y': 99, 'z': 3}, 'b': 10}

    def test_override_dict_with_scalar(self):
        base = {'a': {'x': 1}}
        override = {'a': 'flat_value'}
        result = merge_dicts(base, override)
        assert result == {'a': 'flat_value'}

    def test_does_not_mutate_originals(self):
        base = {'a': {'x': 1}}
        override = {'a': {'y': 2}}
        merge_dicts(base, override)
        assert base == {'a': {'x': 1}}
        assert override == {'a': {'y': 2}}
